@@ -6,7 +6,6 @@ val version_number : string
 val is_native : bool
 val builtin_packages : Ocsigen_lib.StringSet.t
 
-val set_uploaddir : string option -> unit
 val set_logdir : string -> unit
 val set_configfile : string -> unit
 val set_pidfile : string -> unit
@@ -25,7 +24,6 @@ val set_server_timeout : int -> unit
 val set_keepopen_timeout : int -> unit *)
 val set_netbuffersize : int -> unit
 val set_filebuffersize : int -> unit
-val set_maxuploadfilesize : int64 option -> unit
 val set_maxrequestbodysize : int64 option -> unit
 val set_default_charset : string option -> unit
 val set_datadir : string -> unit
@@ -38,11 +36,11 @@ val set_debugmode : bool -> unit
 val set_disablepartialrequests : bool -> unit
 val set_usedefaulthostname : bool -> unit
 val set_respect_pipeline : unit -> unit
-val set_ports : (Unix.inet_addr option * int) list -> unit
-val set_sslports : (Unix.inet_addr option * int) list -> unit
+val set_default_port : int -> unit
+val set_default_sslport : int -> unit
 val set_maxretries : int -> unit
+val set_shutdown_timeout : float option -> unit
 
-val get_uploaddir : unit -> string option
 val get_logdir : unit -> string
 val get_config_file : unit -> string
 val get_pidfile : unit -> string option
@@ -63,7 +61,6 @@ val get_server_timeout : unit -> int
 val get_keepopen_timeout : unit -> int*)
 val get_netbuffersize : unit -> int
 val get_filebuffersize : unit -> int
-val get_maxuploadfilesize : unit -> int64 option
 val get_maxrequestbodysize : unit -> int64 option
 val get_default_charset : unit -> string option
 val get_datadir : unit -> string
@@ -76,10 +73,23 @@ val get_debugmode : unit -> bool
 val get_disablepartialrequests : unit -> bool
 val get_usedefaulthostname : unit -> bool
 val get_respect_pipeline : unit -> bool
-val get_ports : unit -> (Unix.inet_addr option * int) list
-val get_sslports : unit -> (Unix.inet_addr option * int) list
+val get_default_port : unit -> int
+val get_default_sslport : unit -> int
 val get_maxretries : unit -> int
+val get_shutdown_timeout : unit -> float option
 
 val display_version : unit -> 'a
-val config : unit -> Simplexmlparser.xml list
-  
+
+
+(**/**)
+
+(* Global setting for upload directory. This can be overwritten
+   on a per-site basis. Thus, use only the value inside the [ri.request_config]
+   field of a request (which can be changed by the extension
+   [Extendconfiguration]) *)
+val set_uploaddir : string option -> unit
+val get_uploaddir : unit -> string option
+
+(* Same thing for upload size *)
+val set_maxuploadfilesize : int64 option -> unit
+val get_maxuploadfilesize : unit -> int64 option
