@@ -1134,11 +1134,10 @@ let make_uri_components
     match suff with
       | None -> uri
       | Some suff ->
-          if uri = "./" 
-          then 
-            let r = string_of_url_path' suff in
-            if String.length r = 0 then "./" else r
-          else String.concat "/" [uri; string_of_url_path' suff]
+        let suff = string_of_url_path' suff in
+        if uri.[String.length uri - 1] = '/'
+        then uri ^ suff
+        else String.concat "/" [uri; suff]
   in
   let fragment = Ocsigen_lib.apply_option Ocsigen_lib.encode fragment in
   (uri, (params@pregetparams), fragment)
