@@ -31,7 +31,6 @@ open Ocsigen_extensions
 open Eliom_parameters
 open Eliom_services
 
-
 module type FORMCREATE =
   sig
     type form_content_elt
@@ -115,7 +114,7 @@ module type FORMCREATE =
         button_content_elt_list -> button_elt
     val make_textarea :
         ?a:textarea_attrib_t ->
-          name:string -> ?value:pcdata_elt -> rows:int -> cols:int ->
+          name:string -> ?value:string -> rows:int -> cols:int ->
             unit -> textarea_elt
     val make_select :
         ?a:select_attrib_t ->
@@ -149,6 +148,7 @@ module type FORMCREATE =
 module type ELIOMFORMSIG =
   sig
 
+
     type form_content_elt
     type form_content_elt_list
     type form_elt
@@ -167,11 +167,11 @@ module type ELIOMFORMSIG =
     type select_elt
     type select_content_elt
     type select_content_elt_list
+    type option_elt
+    type option_elt_list
     type button_elt
     type button_content_elt
     type button_content_elt_list
-    type option_elt
-    type option_elt_list
 
     type a_attrib_t
     type form_attrib_t
@@ -189,7 +189,11 @@ module type ELIOMFORMSIG =
 
 (** {2 Links and forms} *)
 
-    val make_full_string_uri :
+
+
+    val make_string_uri :
+      ?absolute:bool ->
+      ?absolute_path:bool ->
       ?https:bool ->
       service:('get, unit, [< get_service_kind ],
                [< suff ], 'gn, unit,
@@ -198,11 +202,19 @@ module type ELIOMFORMSIG =
       ?hostname:string ->
       ?port:int ->
       ?fragment:string ->
+      ?keep_nl_params:[ `All | `Persistent | `None ] ->
+      ?nl_params: Eliom_parameters.nl_params_set ->
       'get -> 
       string
-(** Creates the string corresponding to the
-    full (absolute) URL of a service applied to its GET parameters.
+(** Creates the string corresponding to the relative URL of a service applied to
+    its GET parameters.
 
+    If [absolute] is set to [true], or if there is a protocol change,
+    the URL will be absolute.
+    
+    If [absolute_path] is set to [true], and [absolute] is [false],
+    the URL will be absolute, but without [protocol://server:port].
+    
     Default hostname is determined from the [Host] http header of the request
     (or the attribute of <host> tag in
     configuration file if the option [<usedefaulthostname/>] is set).
@@ -210,39 +222,12 @@ module type ELIOMFORMSIG =
     you are switching from or to https).
     But you can choose the hostname or port you want by setting 
     the optional [?hostname] and [?port] parameters here.
- *)
 
-    val make_full_uri :
-      ?https:bool ->
-      service:('get, unit, [< get_service_kind ],
-               [< suff ], 'gn, unit,
-               [< registrable ]) service ->
-      sp:Eliom_sessions.server_params ->
-      ?hostname:string ->
-      ?port:int ->
-      ?fragment:string ->
-      'get -> 
-      uri
-(** Creates the string corresponding to the
-    full (absolute) URL of a service applied to its GET parameters.
- *)
-
-    val make_string_uri :
-      ?https:bool ->
-      service:('get, unit, [< get_service_kind ],
-               [< suff ], 'gn, unit,
-               [< registrable ]) service ->
-      sp:Eliom_sessions.server_params ->
-      ?hostname:string ->
-      ?port:int ->
-      ?fragment:string ->
-      'get -> 
-      string
-(** Creates the string corresponding to the relative URL of a service applied to
-   its GET parameters.
  *)
 
     val make_uri :
+      ?absolute:bool ->
+      ?absolute_path:bool ->
       ?https:bool ->
       service:('get, unit, [< get_service_kind ],
                [< suff ], 'gn, unit,
@@ -251,12 +236,66 @@ module type ELIOMFORMSIG =
       ?hostname:string ->
       ?port:int ->
       ?fragment:string -> 
+      ?keep_nl_params:[ `All | `Persistent | `None ] ->
+      ?nl_params: Eliom_parameters.nl_params_set ->
       'get -> 
       uri
-(** Creates the (relative) URL for a service.
+(** Creates the URL for a service.
     Like the [a] function, it may take extra parameters. *)
 
+    val make_uri_components :
+      ?absolute:bool ->
+      ?absolute_path:bool ->
+      ?https:bool ->
+      service:('get, unit, [< get_service_kind ],
+               [< suff ], 'gn, unit,
+               [< registrable ]) service ->
+      sp:Eliom_sessions.server_params -> 
+      ?hostname:string ->
+      ?port:int ->
+      ?fragment:string -> 
+      ?keep_nl_params:[ `All | `Persistent | `None ] ->
+      ?nl_params: Eliom_parameters.nl_params_set ->
+      'get -> 
+      string * (string * string) list * string option
+(** Creates the URL for a service.
+    Returns the path (as a string, encoded),
+    the association list of get parameters (not encoded),
+    and the fragment (not encoded, if any).
+    Like the [a] function, it may take extra parameters. *)
+
+    val make_post_uri_components :
+      ?absolute:bool ->
+      ?absolute_path:bool ->
+      ?https:bool ->
+      service:('get, 'post, [< post_service_kind ],
+               [< suff ], 'gn, 'pn,
+               [< registrable ]) service ->
+      sp:Eliom_sessions.server_params -> 
+      ?hostname:string ->
+      ?port:int ->
+      ?fragment:string -> 
+      ?keep_nl_params:[ `All | `Persistent | `None ] ->
+      ?nl_params: Eliom_parameters.nl_params_set ->
+      ?keep_get_na_params:bool ->
+      'get -> 
+      'post ->
+      string * (string * string) list * string option * (string * string) list
+(** Like [make_uri_components], but also creates a table of post parameters. *)
+
+    val make_proto_prefix :
+      sp:Eliom_sessions.server_params ->
+      ?hostname:string ->
+      ?port:int ->
+      bool ->
+      string
+(** Creates the string corresponding to the beginning of the URL,
+    containing the scheme (protocol), server and port number (if necessary).
+ *)
+
     val a :
+      ?absolute:bool ->
+      ?absolute_path:bool ->
       ?https:bool ->
       ?a:a_attrib_t ->
       service:('get, unit, [< get_service_kind ],
@@ -266,7 +305,11 @@ module type ELIOMFORMSIG =
       ?hostname:string ->
       ?port:int ->
       ?fragment:string ->
-      a_content_elt_list -> 'get -> a_elt
+      ?keep_nl_params:[ `All | `Persistent | `None ] ->
+      ?nl_params: Eliom_parameters.nl_params_set ->
+      a_content_elt_list -> 
+      'get -> 
+      a_elt
 (** [a service sp cont ()] creates a link to [service].
    The text of
    the link is [cont]. For example [cont] may be something like
@@ -295,6 +338,13 @@ module type ELIOMFORMSIG =
     the optional [?hostname] and [?port] parameters here.
     These options have no effect for relative links.
 
+    You can add non-localized parameters using the optional parameter
+    [nl_params]. See {!Eliom_parameters.nl_params_set}.
+
+    If [~keep_nl_params] is [`Persistent] (resp. [`All]),
+    persistent (resp all) non localized GET parameters
+    will be kept in the URL (default is the default for the service).
+
 *)
 
     val css_link : ?a:link_attrib_t -> uri:uri -> unit -> link_elt
@@ -306,6 +356,8 @@ module type ELIOMFORMSIG =
 
 
     val get_form :
+      ?absolute:bool ->
+      ?absolute_path:bool ->
       ?https:bool ->
       ?a:form_attrib_t ->
       service:('get, unit, [< get_service_kind ],
@@ -315,6 +367,8 @@ module type ELIOMFORMSIG =
       ?hostname:string ->
       ?port:int ->
       ?fragment:string ->
+      ?keep_nl_params:[ `All | `Persistent | `None ] ->
+      ?nl_params: Eliom_parameters.nl_params_set ->
       ('gn -> form_content_elt_list) -> 
       form_elt
 (** [get_form service sp formgen] creates a GET form to [service].
@@ -323,6 +377,8 @@ module type ELIOMFORMSIG =
    of the service parameters as parameters. *)
 
     val lwt_get_form :
+      ?absolute:bool ->
+      ?absolute_path:bool ->
       ?https:bool ->
       ?a:form_attrib_t ->
       service:('get, unit, [< get_service_kind ],
@@ -332,12 +388,16 @@ module type ELIOMFORMSIG =
       ?hostname:string ->
       ?port:int ->
       ?fragment:string ->
+      ?keep_nl_params:[ `All | `Persistent | `None ] ->
+      ?nl_params: Eliom_parameters.nl_params_set ->
       ('gn -> form_content_elt_list Lwt.t) -> 
       form_elt Lwt.t
 (** The same but taking a cooperative function. *)
 
 
     val post_form :
+      ?absolute:bool ->
+      ?absolute_path:bool ->
       ?https:bool ->
       ?a:form_attrib_t ->
       service:('get, 'post, [< post_service_kind ],
@@ -347,13 +407,24 @@ module type ELIOMFORMSIG =
       ?hostname:string ->
       ?port:int ->
       ?fragment:string ->
+      ?keep_nl_params:[ `All | `Persistent | `None ] ->
       ?keep_get_na_params:bool ->
-      ('pn -> form_content_elt_list) -> 'get -> form_elt
+      ?nl_params: Eliom_parameters.nl_params_set ->
+      ('pn -> form_content_elt_list) -> 
+      'get -> 
+      form_elt
 (** [post_form service sp formgen] creates a POST form to [service].
-   The last parameter is for GET parameters (as in the function [a]).
+    The last parameter is for GET parameters (as in the function [a]).
+
+    If [~keep_nl_params] is [`Persistent] (resp. [`All]),
+    persistent (resp all) non localized GET parameters
+    will be kept in the URL (default is the default for the service).
+
  *)
 
     val lwt_post_form :
+      ?absolute:bool ->
+      ?absolute_path:bool ->
       ?https:bool ->
       ?a:form_attrib_t ->
       service:('get, 'post, [< post_service_kind ],
@@ -363,7 +434,9 @@ module type ELIOMFORMSIG =
       ?hostname:string ->
       ?port:int ->
       ?fragment:string ->
+      ?keep_nl_params:[ `All | `Persistent | `None ] ->
       ?keep_get_na_params:bool ->
+      ?nl_params: Eliom_parameters.nl_params_set ->
       ('pn -> form_content_elt_list Lwt.t) -> 
       'get -> 
       form_elt Lwt.t
@@ -371,6 +444,8 @@ module type ELIOMFORMSIG =
 
 
 
+
+
 (** {2 Form widgets} *)
 
 
@@ -404,10 +479,10 @@ module type ELIOMFORMSIG =
              ?value:string -> unit -> input_elt
 (** Creates an [<input>] tag for a string *)
 
-    val user_type_input :
+    val user_type_input : ('a -> string) ->
         ?a:input_attrib_t -> input_type:input_type_t ->
           ?name:[< 'a setoneradio ] param_name ->
-            ?value:'a -> ('a -> string) -> input_elt
+            ?value:'a -> unit -> input_elt
 (** Creates an [<input>] tag for a user type *)
 
     val raw_input :
@@ -419,7 +494,7 @@ module type ELIOMFORMSIG =
 
     val file_input :
         ?a:input_attrib_t ->
-          name:[< file_info setoneradio ] param_name ->
+          name:[< Ocsigen_lib.file_info setoneradio ] param_name ->
             unit -> input_elt
 (** Creates an [<input>] tag for sending a file *)
 
@@ -465,10 +540,10 @@ module type ELIOMFORMSIG =
 (** Creates an [<input type="image" name="..." value="...">] tag that sends
    the coordinates the user clicked on and a value of type string *)
 
-    val user_type_image_input :
+    val user_type_image_input : ('a -> string) ->
         ?a:input_attrib_t ->
           name:[< ('a * coordinates) oneradio ] param_name -> value:'a ->
-            ?src:uri -> ('a -> string) -> input_elt
+            ?src:uri -> unit -> input_elt
 (** Creates an [<input type="image" name="..." value="...">] tag that sends
    the coordinates the user clicked on and a value of user defined type *)
 
@@ -533,10 +608,10 @@ module type ELIOMFORMSIG =
    The service must declare a parameter of type [set].
  *)
 
-    val user_type_checkbox :
+    val user_type_checkbox : ('a -> string) ->
         ?a:input_attrib_t -> ?checked:bool ->
           name:[ `Set of 'a ] param_name -> value:'a ->
-            ('a -> string) -> input_elt
+            unit -> input_elt
 (** Creates a checkbox [<input>] tag that will have a "user type" value.
    Thus you can do several checkboxes with the same name
    (and different values).
@@ -557,9 +632,7 @@ module type ELIOMFORMSIG =
         ?a:input_attrib_t -> ?checked:bool ->
           name:[ `Radio of string ] param_name ->
             value:string -> unit -> input_elt
-(** Creates a radio [<input>] tag with string content.
-    For services taking radio buttons, use [Eliom_parameters.radio].
-*)
+(** Creates a radio [<input>] tag with string content *)
 
     val int_radio :
         ?a:input_attrib_t -> ?checked:bool ->
@@ -585,10 +658,10 @@ module type ELIOMFORMSIG =
              value:float -> unit -> input_elt
 (** Creates a radio [<input>] tag with float content *)
 
-    val user_type_radio :
+    val user_type_radio : ('a -> string) ->
         ?a:input_attrib_t -> ?checked:bool ->
            name:[ `Radio of 'a ] param_name ->
-             value:'a -> ('a -> string) -> input_elt
+             value:'a -> unit -> input_elt
 (** Creates a radio [<input>] tag with user_type content *)
 
     val raw_radio :
@@ -627,9 +700,9 @@ module type ELIOMFORMSIG =
             button_content_elt_list -> button_elt
 (** Creates a [<button>] tag with float content *)
 
-    val user_type_button :
+    val user_type_button : ('a -> string) ->
         ?a:button_attrib_t ->
-          name:[< 'a setone ] param_name -> value:'a -> ('a -> string) ->
+          name:[< 'a setone ] param_name -> value:'a ->
             button_content_elt_list -> button_elt
 (** Creates a [<button>] tag with user_type content *)
 
@@ -651,13 +724,13 @@ module type ELIOMFORMSIG =
 
     val textarea :
         ?a:textarea_attrib_t ->
-          name:[< string setoneradio ] param_name -> ?value:pcdata_elt ->
+          name:[< string setoneradio ] param_name -> ?value:string ->
             rows:int -> cols:int -> unit -> textarea_elt
 (** Creates a [<textarea>] tag *)
 
     val raw_textarea :
         ?a:textarea_attrib_t ->
-          name:string -> ?value:pcdata_elt ->
+          name:string -> ?value:string ->
             rows:int -> cols:int -> unit -> textarea_elt
 (** Creates a [<textarea>] tag for untyped form *)
 
@@ -731,12 +804,11 @@ module type ELIOMFORMSIG =
                 select_elt
 (** Creates a [<select>] tag for string values. *)
 
-    val user_type_select :
+    val user_type_select : ('a -> string) ->
         ?a:select_attrib_t ->
           name:[< `One of 'a ] param_name ->
             'a select_opt ->
               'a select_opt list ->
-                ('a -> string) ->
                   select_elt
 (** Creates a [<select>] tag for user type values. *)
 
@@ -789,16 +861,18 @@ module type ELIOMFORMSIG =
 (** Creates a [<select>] tag for string values. *)
 
     val user_type_multiple_select :
-        ?a:select_attrib_t ->
-          name:[< `Set of 'a ] param_name ->
-            'a select_opt ->
-              'a select_opt list ->
-                ('a -> string) ->
-                  select_elt
+      ('a -> string) ->
+      ?a:select_attrib_t ->
+      name:[< `Set of 'a ] param_name ->
+      'a select_opt ->
+      'a select_opt list ->
+      select_elt
 (** Creates a [<select>] tag for user type values. *)
 
 
-  end
+
+end
+
 
 
 module MakeForms : functor (Pages: FORMCREATE) -> ELIOMFORMSIG with
@@ -841,3 +915,26 @@ and type input_type_t = Pages.input_type_t
 and type button_type_t = Pages.button_type_t
 
 
+(** Constructs a relative link (low level).
+    The first parameter is the current URL,
+    the second is the destination.
+*)
+val reconstruct_relative_url_path : 
+  string list -> string list -> string list
+
+ 
+(**/**)
+
+val make_string_uri :
+  ?absolute:bool ->
+  ?absolute_path:bool ->
+  ?https:bool ->
+  service:('a, 'b, [< Eliom_services.service_kind ],
+           [< Eliom_services.suff ], 'c, 'd, [< Eliom_services.registrable ])
+    Eliom_services.service ->
+  sp:Eliom_sessions.server_params ->
+  ?hostname:string ->
+  ?port:int ->
+  ?fragment:string ->
+  ?keep_nl_params:[ `All | `None | `Persistent ] ->
+  ?nl_params:Eliom_parameters.nl_params_set -> 'a -> string
