@@ -47,9 +47,11 @@ module type REGCREATE =
       ?cookies:Eliom_services.cookie list ->
       ?charset:string ->
       ?code:int ->
+      ?content_type:string ->
+      ?headers: Http_headers.t ->
       sp:Eliom_sessions.server_params ->
       page -> 
-      result_to_send Lwt.t
+      Ocsigen_http_frame.result Lwt.t
 
   end
 
@@ -62,23 +64,31 @@ module type ELIOMREGSIG1 =
     type options
 
     val send :
-        ?options:options ->
-          ?cookies:Eliom_services.cookie list ->
-            ?charset:string ->
-              ?code: int ->
-                sp:Eliom_sessions.server_params ->
-                  page -> result_to_send Lwt.t
+      ?options:options ->
+      ?cookies:Eliom_services.cookie list ->
+      ?charset:string ->
+      ?code: int ->
+      ?content_type:string ->
+      ?headers: Http_headers.t ->
+      sp:Eliom_sessions.server_params ->
+      page -> 
+      Ocsigen_http_frame.result Lwt.t
 
     val register :
-        ?options:options ->
-        ?sp: Eliom_sessions.server_params ->
-        service:('get, 'post,
-                 [< internal_service_kind ],
-                 [< suff ], 'gn, 'pn, [ `Registrable ]) service ->
-        ?error_handler:(Eliom_sessions.server_params ->
-                               (string * exn) list -> page Lwt.t) ->
-        (Eliom_sessions.server_params -> 'get -> 'post -> page Lwt.t) ->
-          unit
+      ?options:options ->
+      ?cookies:Eliom_services.cookie list ->
+      ?charset:string ->
+      ?code: int ->
+      ?content_type:string ->
+      ?headers: Http_headers.t ->
+      ?sp: Eliom_sessions.server_params ->
+      service:('get, 'post,
+               [< internal_service_kind ],
+               [< suff ], 'gn, 'pn, [ `Registrable ]) service ->
+      ?error_handler:(Eliom_sessions.server_params ->
+                        (string * exn) list -> page Lwt.t) ->
+      (Eliom_sessions.server_params -> 'get -> 'post -> page Lwt.t) ->
+      unit
 (** registers an service in the public service table
    with the associated handler function.
    [register service t f] will associate the service [service]
@@ -114,9 +124,13 @@ module type ELIOMREGSIG1 =
 
  *)
 
-
     val register_for_session :
       ?options:options ->
+      ?cookies:Eliom_services.cookie list ->
+      ?charset:string ->
+      ?code: int ->
+      ?content_type:string ->
+      ?headers: Http_headers.t ->
       ?session_name:string ->
       ?secure:bool ->
       sp:Eliom_sessions.server_params ->
@@ -124,29 +138,47 @@ module type ELIOMREGSIG1 =
                [< suff ], 'gn, 'pn, [ `Registrable ]) service ->
       ?error_handler:(Eliom_sessions.server_params -> (string * exn) list ->
                         page Lwt.t) ->
-      (Eliom_sessions.server_params -> 'get -> 'post -> page Lwt.t) -> unit
+      (Eliom_sessions.server_params -> 'get -> 'post -> page Lwt.t) -> 
+      unit
 (** registers a handler for a service in the session table.
-   If the same client does a request to this service, this function will be
-   used instead of the one from the public table.
+    If the same client does a request to this service, this function will be
+    used instead of the one from the public table.
 
-   If [~secure] is false when the protocol is https, it will affect
-   the unsecure session, otherwise, il will affect the secure session in 
-   https, the unsecure one in http.
+    Warning:
+    - All main services created during initialization must be
+    registered in the public table during initialisation,
+    but never after,
+    - You can't register a service in a session table
+    when no session is active (i.e. outside a service handler, 
+    when you do not have sp)
+    
+    [?session_name] is the name of the session, if you want several
+    service sessions on the same site.
+    
+    If [~secure] is false when the protocol is https, the service will be 
+    registered in the unsecure session, 
+    otherwise in the secure session with https, the unsecure one with http.
+    (Secure session means that Eliom will ask the browser to send the cookie
+    only through HTTPS).
 
-   Warning:
-   - All main services created during initialization must be
-   registered in the public table during initialisation,
-   but never after,
-   - You can't register a service in a session table
-   when no session is active (i.e. outside a service handler)
+    Note that in the case of CSRF safe coservices, parameters
+    [?session_name] and [?secure] must match exactly the session name
+    and secure option specified while creating the CSRF safe service. 
+    Otherwise, the registration will fail
+    with {Eliom_services.Wrong_session_table_for_CSRF_safe_coservice}
  *)
 
 
     val register_new_service :
       ?options:options ->
+      ?cookies:Eliom_services.cookie list ->
+      ?charset:string ->
+      ?code: int ->
+      ?content_type:string ->
+      ?headers: Http_headers.t ->
       ?sp: Eliom_sessions.server_params ->
       ?https:bool ->
-      path:url_path ->
+      path:Ocsigen_lib.url_path ->
       get_params:('get, [< suff ] as 'tipo, 'gn) params_type ->
       ?error_handler:(Eliom_sessions.server_params -> (string * exn) list ->
                         page Lwt.t) ->
@@ -160,8 +192,16 @@ module type ELIOMREGSIG1 =
 
     val register_new_coservice :
       ?options:options ->
+      ?cookies:Eliom_services.cookie list ->
+      ?charset:string ->
+      ?code: int ->
+      ?content_type:string ->
+      ?headers: Http_headers.t ->
       ?sp: Eliom_sessions.server_params ->
       ?name: string ->
+      ?csrf_safe: bool ->
+      ?csrf_session_name: string ->
+      ?csrf_secure_session: bool ->
       ?max_use:int ->
       ?timeout:float ->
       ?https:bool ->
@@ -185,8 +225,16 @@ module type ELIOMREGSIG1 =
 
     val register_new_coservice' :
       ?options:options ->
+      ?cookies:Eliom_services.cookie list ->
+      ?charset:string ->
+      ?code: int ->
+      ?content_type:string ->
+      ?headers: Http_headers.t ->
       ?sp: Eliom_sessions.server_params ->
       ?name: string ->
+      ?csrf_safe: bool ->
+      ?csrf_session_name: string ->
+      ?csrf_secure_session: bool ->
       ?max_use:int ->
       ?timeout:float ->
       ?https:bool ->
@@ -203,10 +251,16 @@ module type ELIOMREGSIG1 =
 
     val register_new_coservice_for_session :
       ?options:options ->
+      ?cookies:Eliom_services.cookie list ->
+      ?charset:string ->
+      ?code: int ->
+      ?content_type:string ->
+      ?headers: Http_headers.t ->
       ?session_name:string ->
       ?secure:bool ->
       sp:Eliom_sessions.server_params ->
       ?name: string ->
+      ?csrf_safe: bool ->
       ?max_use:int ->
       ?timeout:float ->
       ?https:bool ->
@@ -230,10 +284,16 @@ module type ELIOMREGSIG1 =
 
     val register_new_coservice_for_session' :
       ?options:options ->
+      ?cookies:Eliom_services.cookie list ->
+      ?charset:string ->
+      ?code: int ->
+      ?content_type:string ->
+      ?headers: Http_headers.t ->
       ?session_name:string ->
       ?secure:bool ->
       sp:Eliom_sessions.server_params ->
       ?name: string ->
+      ?csrf_safe: bool ->
       ?max_use:int ->
       ?timeout:float ->
       ?https:bool ->
@@ -250,11 +310,17 @@ module type ELIOMREGSIG1 =
 
     val register_new_post_service :
       ?options:options ->
+      ?cookies:Eliom_services.cookie list ->
+      ?charset:string ->
+      ?code: int ->
+      ?content_type:string ->
+      ?headers: Http_headers.t ->
       ?sp: Eliom_sessions.server_params ->
       ?https:bool ->
       fallback:('get, unit,
-                [ `Attached of [ `Internal of
-                                   ([ `Service | `Coservice ] as 'kind) * [`Get] ] a_s ],
+                [ `Attached of
+                    [ `Internal of
+                        ([ `Service | `Coservice ] as 'kind) * [`Get] ] a_s ],
                 [< suff ] as 'tipo, 'gn,
                 unit, [< `Registrable ])
         service ->
@@ -270,8 +336,16 @@ module type ELIOMREGSIG1 =
 
     val register_new_post_coservice :
       ?options:options ->
+      ?cookies:Eliom_services.cookie list ->
+      ?charset:string ->
+      ?code: int ->
+      ?content_type:string ->
+      ?headers: Http_headers.t ->
       ?sp: Eliom_sessions.server_params ->
       ?name: string ->
+      ?csrf_safe: bool ->
+      ?csrf_session_name: string ->
+      ?csrf_secure_session: bool ->
       ?max_use:int ->
       ?timeout:float ->
       ?https:bool ->
@@ -294,8 +368,16 @@ module type ELIOMREGSIG1 =
 
     val register_new_post_coservice' :
       ?options:options ->
+      ?cookies:Eliom_services.cookie list ->
+      ?charset:string ->
+      ?code: int ->
+      ?content_type:string ->
+      ?headers: Http_headers.t ->
       ?sp: Eliom_sessions.server_params ->
       ?name: string ->
+      ?csrf_safe: bool ->
+      ?csrf_session_name: string ->
+      ?csrf_secure_session: bool ->
       ?max_use:int ->
       ?timeout:float ->
       ?keep_get_na_params:bool ->
@@ -313,8 +395,12 @@ module type ELIOMREGSIG1 =
 (*
     val register_new_get_post_coservice' :
         ?options:options ->
+      ?cookies:Eliom_services.cookie list ->
+      ?charset:string ->
+      ?code: int ->
+      ?content_type:string ->
+      ?headers: Http_headers.t ->
         ?sp: Eliom_sessions.server_params ->
-      ?name: string ->
         ?max_use:int ->
         ?timeout:float ->
       ?https:bool ->
@@ -335,10 +421,16 @@ module type ELIOMREGSIG1 =
 
     val register_new_post_coservice_for_session :
       ?options:options ->
+      ?cookies:Eliom_services.cookie list ->
+      ?charset:string ->
+      ?code: int ->
+      ?content_type:string ->
+      ?headers: Http_headers.t ->
       ?session_name:string ->
       ?secure:bool ->
       sp:Eliom_sessions.server_params ->
       ?name: string ->
+      ?csrf_safe: bool ->
       ?max_use:int ->
       ?timeout:float ->
       ?https:bool ->
@@ -361,10 +453,16 @@ module type ELIOMREGSIG1 =
 
     val register_new_post_coservice_for_session' :
       ?options:options ->
+      ?cookies:Eliom_services.cookie list ->
+      ?charset:string ->
+      ?code: int ->
+      ?content_type:string ->
+      ?headers: Http_headers.t ->
       ?session_name:string ->
       ?secure:bool ->
       sp:Eliom_sessions.server_params ->
       ?name: string ->
+      ?csrf_safe: bool ->
       ?max_use:int ->
       ?timeout:float ->
       ?keep_get_na_params:bool ->
@@ -383,9 +481,10 @@ module type ELIOMREGSIG1 =
     val register_new_get_post_coservice_for_session' :
         ?options:options ->
         ?session_name:string ->
-      ?secure:bool ->
+  ?secure:bool ->
         sp:Eliom_sessions.server_params ->
-      ?name: string ->
+  ?name: string ->
+      ?csrf_safe: bool ->
         ?max_use:int ->
         ?timeout:float ->
       ?https:bool ->
@@ -403,6 +502,8 @@ module type ELIOMREGSIG1 =
 (* * Same as [new_get_post_coservice] followed by [register_for_session] *)
 *)
 
+
+
   end
 
 
