@@ -187,7 +187,7 @@ let reconstruct_params
   in
   let aux2 typ params =
     let rec aux_list t params files name pref suff =
-      let rec aa i lp fl pref suff =
+      let rec aa i lp fl pref =
         let rec end_of_list len = function
           | [] -> true
           | (a,_)::_ when
@@ -199,18 +199,18 @@ let reconstruct_params
         then Res_ ((Obj.magic []), lp, fl)
         else
           try
-            match aux t lp fl pref (suff^(make_list_suffix i)) with
+            match aux t lp fl pref (make_list_suffix i) with
               | Res_ (v,lp2,f) ->
-                  (match aa (i+1) lp2 f pref suff with
+                  (match aa (i+1) lp2 f pref with
                      | Res_ (v2,lp3,f2) -> Res_ ((Obj.magic (v::v2)),lp3,f2)
                      | err -> err)
               | Errors_ (errs, l, f) ->
-                  (match aa (i+1) l f pref suff with
+                  (match aa (i+1) l f pref with
                      | Res_ (_,ll,ff) -> Errors_ (errs, ll, ff)
                      | Errors_ (errs2, ll, ff) -> Errors_ ((errs@errs2), ll, ff))
           with Not_found -> Res_ ((Obj.magic []), lp, files)
       in
-      aa 0 params files (pref^name^".") suff
+      aa 0 params files (pref^name^suff^".")
     and aux (typ : ('a, [<`WithSuffix|`WithoutSuffix|`Endsuffix], 'b) params_type)
         params files pref suff : 'a res_reconstr_param =
       match typ with
