@@ -28,7 +28,6 @@
 open XHTML.M
 open Xhtmltypes
 open Ocsigen_extensions
-open Eliom_sessions
 open Eliom_services
 open Eliom_parameters
 open Eliom_mkforms
@@ -50,12 +49,26 @@ module type XHTMLFORMSSIG = sig
 
 
 
+
+
   open XHTML.M
   open Xhtmltypes
 
 (** {2 Links and forms} *)
 
-    val make_full_string_uri :
+
+
+
+
+
+
+
+
+
+
+    val make_string_uri :
+      ?absolute:bool ->
+      ?absolute_path:bool ->
       ?https:bool ->
       service:('get, unit, [< get_service_kind ],
                [< suff ], 'gn, unit,
@@ -64,11 +77,19 @@ module type XHTMLFORMSSIG = sig
       ?hostname:string ->
       ?port:int ->
       ?fragment:string ->
+      ?keep_nl_params:[ `All | `Persistent | `None ] ->
+      ?nl_params: Eliom_parameters.nl_params_set ->
       'get -> 
       string
-(** Creates the string corresponding to the
-    full (absolute) URL of a service applied to its GET parameters.
+(** Creates the string corresponding to the relative URL of a service applied to
+    its GET parameters.
 
+    If [absolute] is set to [true], or if there is a protocol change,
+    the URL will be absolute.
+    
+    If [absolute_path] is set to [true], and [absolute] is [false],
+    the URL will be absolute, but without [protocol://server:port].
+    
     Default hostname is determined from the [Host] http header of the request
     (or the attribute of <host> tag in
     configuration file if the option [<usedefaulthostname/>] is set).
@@ -77,39 +98,11 @@ module type XHTMLFORMSSIG = sig
     But you can choose the hostname or port you want by setting 
     the optional [?hostname] and [?port] parameters here.
 
- *)
-
-    val make_full_uri :
-      ?https:bool ->
-      service:('get, unit, [< get_service_kind ],
-               [< suff ], 'gn, unit,
-               [< registrable ]) service ->
-      sp:Eliom_sessions.server_params ->
-      ?hostname:string ->
-      ?port:int ->
-      ?fragment:string ->
-      'get -> 
-      XHTML.M.uri
-(** Creates the string corresponding to the
-    full (absolute) URL of a service applied to its GET parameters.
- *)
-
-    val make_string_uri :
-      ?https:bool ->
-      service:('get, unit, [< get_service_kind ],
-               [< suff ], 'gn, unit,
-               [< registrable ]) service ->
-      sp:Eliom_sessions.server_params ->
-      ?hostname:string ->
-      ?port:int ->
-      ?fragment:string ->
-      'get -> 
-      string
-(** Creates the string corresponding to the relative URL of a service applied to
-   its GET parameters.
  *)
 
     val make_uri :
+      ?absolute:bool ->
+      ?absolute_path:bool ->
       ?https:bool ->
       service:('get, unit, [< get_service_kind ],
                [< suff ], 'gn, unit,
@@ -118,37 +111,89 @@ module type XHTMLFORMSSIG = sig
       ?hostname:string ->
       ?port:int ->
       ?fragment:string -> 
+      ?keep_nl_params:[ `All | `Persistent | `None ] ->
+      ?nl_params: Eliom_parameters.nl_params_set ->
       'get -> 
       uri
-(** Create the text of the service. Like the [a] function, it may take
-   extra parameters. *)
+(** Creates the URL for a service.
+    Like the [a] function, it may take extra parameters. *)
 
+    val make_uri_components :
+      ?absolute:bool ->
+      ?absolute_path:bool ->
+      ?https:bool ->
+      service:('get, unit, [< get_service_kind ],
+               [< suff ], 'gn, unit,
+               [< registrable ]) service ->
+      sp:Eliom_sessions.server_params -> 
+      ?hostname:string ->
+      ?port:int ->
+      ?fragment:string -> 
+      ?keep_nl_params:[ `All | `Persistent | `None ] ->
+      ?nl_params: Eliom_parameters.nl_params_set ->
+      'get -> 
+      string * (string * string) list * string option
+(** Creates the URL for a service.
+    Returns the path (as a string, encoded),
+    the association list of get parameters (not encoded),
+    and the fragment (not encoded, if any).
+    Like the [a] function, it may take extra parameters. *)
 
+    val make_post_uri_components :
+      ?absolute:bool ->
+      ?absolute_path:bool ->
+      ?https:bool ->
+      service:('get, 'post, [< post_service_kind ],
+               [< suff ], 'gn, 'pn,
+               [< registrable ]) service ->
+      sp:Eliom_sessions.server_params -> 
+      ?hostname:string ->
+      ?port:int ->
+      ?fragment:string -> 
+      ?keep_nl_params:[ `All | `Persistent | `None ] ->
+      ?nl_params: Eliom_parameters.nl_params_set ->
+      ?keep_get_na_params:bool ->
+      'get -> 
+      'post ->
+      string * (string * string) list * string option * (string * string) list
+(** Like [make_uri_components], but also creates a table of post parameters. *)
 
-  val a :
-    ?https:bool ->
-    ?a:a_attrib attrib list ->
-    service:
-      ('get, unit, [< get_service_kind ],
-       [< suff ], 'gn, 'pn,
-       [< registrable ]) service ->
-    sp:Eliom_sessions.server_params -> 
-    ?hostname:string ->
-    ?port:int ->
-    ?fragment:string ->
-    a_content elt list -> 
-    'get -> 
+    val make_proto_prefix :
+      sp:Eliom_sessions.server_params ->
+      ?hostname:string ->
+      ?port:int ->
+      bool ->
+      string
+(** Creates the string corresponding to the beginning of the URL,
+    containing the scheme (protocol), server and port number (if necessary).
+ *)
+
+    val a :
+      ?absolute:bool ->
+      ?absolute_path:bool ->
+      ?https:bool ->
+      ?a:a_attrib attrib list ->
+      service:('get, unit, [< get_service_kind ],
+               [< suff ], 'gn, 'pn,
+               [< registrable ]) service ->
+      sp:Eliom_sessions.server_params -> 
+      ?hostname:string ->
+      ?port:int ->
+      ?fragment:string ->
+      ?keep_nl_params:[ `All | `Persistent | `None ] ->
+      ?nl_params: Eliom_parameters.nl_params_set ->
+      a_content elt list -> 
+      'get -> 
     [> a] XHTML.M.elt
 (** [a service sp cont ()] creates a link to [service].
    The text of
    the link is [cont]. For example [cont] may be something like
-   [[pcdata "click here"]].
+   [\[pcdata "click here"\]].
 
    The last  parameter is for GET parameters.
    For example [a service sp cont (42,"hello")]
 
-   The [~a] optional parameter is used for extra attributes
-   (see the module XHTML.M).
+   The [~a] optional parameter is used for extra attributes.
 
    The [~fragment] optional parameter is used for the "fragment" part
    of the URL, that is, the part after character "#".
@@ -168,18 +213,26 @@ module type XHTMLFORMSSIG = sig
     the optional [?hostname] and [?port] parameters here.
     These options have no effect for relative links.
 
- *)
+    You can add non-localized parameters using the optional parameter
+    [nl_params]. See {!Eliom_parameters.nl_params_set}.
 
-  val css_link : ?a:(link_attrib attrib list) ->
-    uri:uri -> unit ->[> link ] elt
+    If [~keep_nl_params] is [`Persistent] (resp. [`All]),
+    persistent (resp all) non localized GET parameters
+    will be kept in the URL (default is the default for the service).
+
+*)
+
+    val css_link : ?a:link_attrib attrib list -> uri:uri -> unit -> [>link] elt
 (** Creates a [<link>] tag for a Cascading StyleSheet (CSS). *)
 
-  val js_script : ?a:(script_attrib attrib list) ->
-    uri:uri -> unit -> [> script ] elt
+    val js_script :
+        ?a:script_attrib attrib list -> uri:uri -> unit -> [>script] elt
 (** Creates a [<script>] tag to add a javascript file *)
 
 
     val get_form :
+      ?absolute:bool ->
+      ?absolute_path:bool ->
       ?https:bool ->
       ?a:form_attrib attrib list ->
       service:('get, unit, [< get_service_kind ],
@@ -189,14 +242,18 @@ module type XHTMLFORMSSIG = sig
       ?hostname:string ->
       ?port:int ->
       ?fragment:string ->
+      ?keep_nl_params:[ `All | `Persistent | `None ] ->
+      ?nl_params: Eliom_parameters.nl_params_set ->
       ('gn -> form_content elt list) -> 
-      [> form ] elt
+      [>form] elt
 (** [get_form service sp formgen] creates a GET form to [service].
    The content of
    the form is generated by the function [formgen], that takes the names
    of the service parameters as parameters. *)
 
     val lwt_get_form :
+      ?absolute:bool ->
+      ?absolute_path:bool ->
       ?https:bool ->
       ?a:form_attrib attrib list ->
       service:('get, unit, [< get_service_kind ],
@@ -206,19 +263,16 @@ module type XHTMLFORMSSIG = sig
       ?hostname:string ->
       ?port:int ->
       ?fragment:string ->
+      ?keep_nl_params:[ `All | `Persistent | `None ] ->
+      ?nl_params: Eliom_parameters.nl_params_set ->
       ('gn -> form_content elt list Lwt.t) -> 
-      form elt Lwt.t
-(** The same but taking a cooperative function. 
-    Use it like this:
-
-    [Eliom_predefmod.Xhtml.lwt_get_form ~service:... ~sp f >>= fun form ->
-    let form = 
-      (form : Xhtmltypes.form XHTML.M.elt :> [> Xhtmltypes.form ] XHTML.M.elt)
-    in ...]
-*)
+      [>form] elt Lwt.t
+(** The same but taking a cooperative function. *)
 
 
     val post_form :
+      ?absolute:bool ->
+      ?absolute_path:bool ->
       ?https:bool ->
       ?a:form_attrib attrib list ->
       service:('get, 'post, [< post_service_kind ],
@@ -228,16 +282,24 @@ module type XHTMLFORMSSIG = sig
       ?hostname:string ->
       ?port:int ->
       ?fragment:string ->
+      ?keep_nl_params:[ `All | `Persistent | `None ] ->
       ?keep_get_na_params:bool ->
+      ?nl_params: Eliom_parameters.nl_params_set ->
       ('pn -> form_content elt list) -> 
       'get -> 
       [>form] elt
 (** [post_form service sp formgen] creates a POST form to [service].
-   The last parameter is for GET parameters (as in the function [a]).
+    The last parameter is for GET parameters (as in the function [a]).
+
+    If [~keep_nl_params] is [`Persistent] (resp. [`All]),
+    persistent (resp all) non localized GET parameters
+    will be kept in the URL (default is the default for the service).
+
  *)
 
-
     val lwt_post_form :
+      ?absolute:bool ->
+      ?absolute_path:bool ->
       ?https:bool ->
       ?a:form_attrib attrib list ->
       service:('get, 'post, [< post_service_kind ],
@@ -247,18 +309,15 @@ module type XHTMLFORMSSIG = sig
       ?hostname:string ->
       ?port:int ->
       ?fragment:string ->
+      ?keep_nl_params:[ `All | `Persistent | `None ] ->
       ?keep_get_na_params:bool ->
+      ?nl_params: Eliom_parameters.nl_params_set ->
       ('pn -> form_content elt list Lwt.t) -> 
       'get -> 
-      form elt Lwt.t
-(** The same but taking a cooperative function. 
-    Use it like this:
+      [>form] elt Lwt.t
+(** The same but taking a cooperative function. *)
+
 
-    [Eliom_predefmod.Xhtml.lwt_post_form ~service:... ~sp f >>= fun form ->
-    let form = 
-      (form : Xhtmltypes.form XHTML.M.elt :> [> Xhtmltypes.form ] XHTML.M.elt)
-    in ...]
-*)
 
 
 
@@ -304,9 +363,10 @@ module type XHTMLFORMSSIG = sig
 (** Creates an [<input>] tag for a string *)
 
   val user_type_input :
+    ('a -> string) -> 
       ?a:input_attrib attrib list -> input_type:[< basic_input_type ] ->
         ?name:[< 'a setoneradio ] param_name ->
-          ?value:'a -> ('a -> string) -> [> input ] elt
+          ?value:'a -> unit -> [> input ] elt
 (** Creates an [<input>] tag for a user type *)
 
   val raw_input :
@@ -319,7 +379,7 @@ module type XHTMLFORMSSIG = sig
 
   val file_input :
       ?a:input_attrib attrib list ->
-        name:[< file_info setoneradio ] param_name ->
+        name:[< Ocsigen_lib.file_info setoneradio ] param_name ->
           unit -> [> input ] elt
 (** Creates an [<input>] tag for sending a file *)
 
@@ -366,9 +426,10 @@ module type XHTMLFORMSSIG = sig
    the coordinates the user clicked on and a value of type string *)
 
   val user_type_image_input :
+    ('a -> string) -> 
       ?a:input_attrib attrib list ->
         name:[< ('a * coordinates) oneradio ] param_name -> value:'a ->
-          ?src:uri -> ('a -> string) -> [> input ] elt
+          ?src:uri -> unit -> [> input ] elt
 (** Creates an [<input type="image" name="..." value="...">] tag that sends
    the coordinates the user clicked on and a value of user defined type *)
 
@@ -439,9 +500,10 @@ module type XHTMLFORMSSIG = sig
 
 
     val user_type_checkbox :
+      ('a -> string) -> 
         ?a:input_attrib attrib list -> ?checked:bool ->
-          name:[ `Set of 'a ] param_name -> value:'a ->
-            ('a -> string) -> [> input ] elt
+          name:[ `Set of 'a ] param_name -> value:'a -> unit -> 
+            [> input ] elt
 (** Creates a checkbox [<input>] tag that will have a "user type" value.
    Thus you can do several checkboxes with the same name
    (and different values).
@@ -481,8 +543,9 @@ module type XHTMLFORMSSIG = sig
      name:[ `Radio of float ] param_name -> value:float -> unit -> [> input ] elt
 (** Creates a radio [<input>] tag with float content *)
 
-  val user_type_radio : ?a:(input_attrib attrib list ) -> ?checked:bool ->
-    name:[ `Radio of 'a ] param_name -> value:'a -> ('a -> string) -> [> input ] elt
+  val user_type_radio : 
+    ('a -> string) -> ?a:(input_attrib attrib list ) -> ?checked:bool ->
+    name:[ `Radio of 'a ] param_name -> value:'a -> unit -> [> input ] elt
 (** Creates a radio [<input>] tag with user_type content *)
 
   val raw_radio : ?a:(input_attrib attrib list ) -> ?checked:bool ->
@@ -518,8 +581,8 @@ module type XHTMLFORMSSIG = sig
       button_content elt list -> [> button ] elt
 (** Creates a [<button>] tag with float content *)
 
-  val user_type_button : ?a:button_attrib attrib list ->
-    name:[< 'a setone ] param_name -> value:'a -> ('a -> string) ->
+  val user_type_button : ('a -> string) -> ?a:button_attrib attrib list ->
+    name:[< 'a setone ] param_name -> value:'a ->
       button_content elt list -> [> button ] elt
 (** Creates a [<button>] tag with user_type content *)
 
@@ -539,7 +602,7 @@ module type XHTMLFORMSSIG = sig
   val textarea :
       ?a:textarea_attrib attrib list ->
         name:[< string setoneradio ] param_name ->
-          ?value:Xhtmltypes.pcdata XHTML.M.elt ->
+          ?value:string ->
             rows:int -> cols:int ->
               unit -> [> textarea ] elt
 (** Creates a [<textarea>] tag *)
@@ -547,7 +610,7 @@ module type XHTMLFORMSSIG = sig
   val raw_textarea :
       ?a:textarea_attrib attrib list ->
         name:string ->
-          ?value:Xhtmltypes.pcdata XHTML.M.elt ->
+          ?value:string ->
             rows:int -> cols:int ->
               unit -> [> textarea ] elt
 (** Creates a [<textarea>] tag for untyped form *)
@@ -615,11 +678,11 @@ module type XHTMLFORMSSIG = sig
 (** Creates a [<select>] tag for string values. *)
 
   val user_type_select :
+    ('a -> string) -> 
       ?a:select_attrib attrib list ->
         name:[< `One of 'a ] param_name ->
           'a select_opt ->
             'a select_opt list ->
-              ('a -> string) ->
                 [> select ] elt
 (** Creates a [<select>] tag for user type values. *)
 
@@ -673,11 +736,11 @@ module type XHTMLFORMSSIG = sig
 (** Creates a [<select>] tag for string values. *)
 
   val user_type_multiple_select :
+    ('a -> string) -> 
       ?a:select_attrib attrib list ->
         name:[< `Set of 'a ] param_name ->
           'a select_opt ->
             'a select_opt list ->
-              ('a -> string) ->
                 [> select ] elt
 (** Creates a [<select>] tag for user type values. *)
 
@@ -692,6 +755,7 @@ module type XHTMLFORMSSIG = sig
 
 end
 
+
 (** {3 Forms and registration functions} *)
 
       (** Eliom forms and service registration functions for XHTML *)
@@ -799,26 +863,21 @@ module Text : Eliom_mkreg.ELIOMREGSIG with type page = string * string
     then the page corresponding to the URL (without POST parameters
     or non-attached parameters or coservice parameters) is sent to the browser.
 
-   Actions return a list of exceptions.
-   You may use this to give information to the handler that will be called
-   to reload the page.
-   Use {!Eliom_sessions.get_exn} to access these exceptions from this handler.
+    If you want to give information to the handler that will be called
+    to reload the page, put it in the polymorphic table returned by 
+    {!Eliom_sessions.get_request_cache}.
 
     If you give the optional parameter
     [~options:`NoReload] to the registration function, no page will be sent.
  *)
 module Action : Eliom_mkreg.ELIOMREGSIG with
-  type page = exn list
+  type page = unit
   and type options = [ `Reload | `NoReload ]
 
-(** The same, for backwards compatibility *)
-module Actions : Eliom_mkreg.ELIOMREGSIG with
-  type page = exn list
-  and type options = [ `Reload | `NoReload ]
 
 (** Like actions, but the page is not reloaded. Just do something and do
    not generate any page. To be used carefully. Probably not usefull at all.
-   (Same as {!Eliom_predefmod.Actions} with [`NoReload] option).
+   (Same as {!Eliom_predefmod.Action} with [`NoReload] option).
  *)
 module Unit : Eliom_mkreg.ELIOMREGSIG with
   type page = unit
@@ -865,7 +924,7 @@ module Files : Eliom_mkreg.ELIOMREGSIG with
    .
  *)
 module Any : Eliom_mkreg.ELIOMREGSIG with
-  type page = Eliom_services.result_to_send
+  type page = Ocsigen_http_frame.result
 
 (** Allows to send raw data using Ocsigen's streams.
     The content is a pair conatining:
