@@ -19,23 +19,26 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *)
 
-
-
 open Lwt
 open Ocsigen_lib
-open Ocsigen_extensions
 open Eliom_sessions
 open Eliom_parameters
 open Lazy
 
 
+(* Manipulation of services - this code can be use only on server side. *)
+
+include Eliom_services_obrowser
+
+exception Wrong_session_table_for_CSRF_safe_coservice
+
 (** Type used for cookies to set.
     The float option is the timestamp for the expiration date.
     The strings are names and values.
  *)
 type cookie = Eliom_common.cookie =
-  | Set of url_path option * float option * string * string * bool
-  | Unset of url_path option * string
+  | Set of Ocsigen_lib.url_path option * float option * string * string * bool
+  | Unset of Ocsigen_lib.url_path option * string
 
 let cookie_table_of_eliom_cookies
     ?(oldtable= Ocsigen_http_frame.Cookies.empty) ~sp cl =
@@ -46,14 +49,6 @@ let cookie_table_of_eliom_cookies
 
 
 
-(** The type to send if you want to create your own modules for generating
-   pages
- *)
-type result_to_send = Eliom_common.result_to_send =
-  | EliomResult of Ocsigen_http_frame.result
-  | EliomExn of (exn list * cookie list)
-
-
 
 (** This function may be used for services that cannot be interrupted
   (no cooperation point for threads). It is defined by
@@ -62,104 +57,10 @@ type result_to_send = Eliom_common.result_to_send =
 let sync f sp g p = Lwt.return (f sp g p)
 
 
-(** Typed services *)
-type suff = [ `WithSuffix | `WithoutSuffix ]
 
-type servcoserv = [ `Service | `Coservice ]
-type getpost = [ `Get | `Post ]
-      (* `Post means that there is at least one post param
-         (possibly only the state post param).
-         `Get is for all the other cases.
-       *)
-type attached_service_kind =
-    [ `Internal of servcoserv * getpost
-    | `External]
-
-type get_attached_service_kind =
-    [ `Internal of servcoserv * [ `Get ]
-    | `External ]
-
-type post_attached_service_kind =
-    [ `Internal of servcoserv * [ `Post ]
-    | `External ]
-
-type internal =
-    [ `Internal of servcoserv * getpost ]
-
-type registrable = [ `Registrable | `Unregistrable ]
-
-type +'a a_s =
-    {prefix: string; (* name of the server and protocol,
-                        for external links. Ex: http://ocsigen.org *)
-     subpath: url_path; (* name of the service without parameters *)
-     fullpath: url_path; (* full path of the service = site_dir@subpath *)
-     att_kind: 'a; (* < attached_service_kind *)
-     get_name: Eliom_common.att_key;
-     post_name: Eliom_common.att_key;
-   }
-
-type +'a na_s =
-    {na_name: Eliom_common.na_key;
-     na_kind: [ `Get | `Post of bool ]
-       (*
-          where bool is "keep_get_na_params":
-          do we keep GET non-attached parameters in links (if any)
-          (31/12/2007 - experimental -
-          WAS: 'a, but may be removed (was not used))
-       *)
-    }
-
-type service_kind =
-    [ `Attached of attached_service_kind a_s
-    | `Nonattached of getpost na_s ]
-
-type internal_service_kind =
-    [ `Attached of internal a_s
-    | `Nonattached of getpost na_s ]
-
-type get_service_kind =
-    [ `Attached of get_attached_service_kind a_s
-    | `Nonattached of [ `Get ] na_s ]
-
-type post_service_kind =
-    [ `Attached of post_attached_service_kind a_s
-    | `Nonattached of [ `Post ] na_s ]
-
-type attached =
-    [ `Attached of attached_service_kind a_s ]
-
-type nonattached =
-    [ `Nonattached of getpost na_s ]
-
-type ('get,'post,+'kind,+'tipo,+'getnames,+'postnames,+'registr) service =
-    {
-     pre_applied_parameters: (string * string) list;
-     get_params_type: ('get, 'tipo, 'getnames) params_type;
-     post_params_type: ('post, [`WithoutSuffix], 'postnames) params_type;
-     max_use: int option; (* Max number of use of this service *)
-     timeout: float option; (* Timeout for this service (the service will
-          disappear if it has not been used during this amount of seconds) *)
-     kind: 'kind; (* < service_kind *)
-     https: bool; (* force https *)
-   }
-
-let get_kind_ s = s.kind
-let get_att_kind_ s = s.att_kind
-let get_pre_applied_parameters_ s = s.pre_applied_parameters
-let get_get_params_type_ s = s.get_params_type
-let get_post_params_type_ s = s.post_params_type
-let get_prefix_ s = s.prefix
-let get_sub_path_ s = s.subpath
-let get_full_path_ s = s.fullpath
-let get_get_name_ s = s.get_name
-let get_post_name_ s = s.post_name
-let get_na_name_ s = s.na_name
-let get_na_kind_ s = s.na_kind
-let get_max_use_ s = s.max_use
-let get_timeout_ s = s.timeout
-let get_https s = s.https
-
-let new_state =
+(**********)
+let new_state = Eliommod_cookies.make_new_cookie_value
+(* WAS:
   (* This does not need to be cryptographickly robust.
      We just want to avoid the same values when the server is relaunched.
    *)
@@ -168,6 +69,15 @@ let new_state =
     c := Int64.add !c Int64.one ;
     (Printf.sprintf "%x" (Random.int 0xFFFF))^(Printf.sprintf "%Lx" !c)
 
+   But I turned this into cryptographickly robust version
+   to implement CSRF-safe services.
+*)
+
+let get_or_post = function
+  | `Internal (_, `Get) -> Ocsigen_http_frame.Http_header.GET
+  | _ -> Ocsigen_http_frame.Http_header.POST
+(*  | `External -> POST ? *)
+
 
 (*****************************************************************************)
 (*****************************************************************************)
@@ -184,56 +94,9 @@ let register_eliom_module name f =
 (*****************************************************************************)
 (*****************************************************************************)
 
-(** Satic directories **)
-let static_dir_ ?(https = false) ~sp () =
-    {
-     pre_applied_parameters = [];
-     get_params_type = suffix (all_suffix Eliom_common.eliom_suffix_name);
-     post_params_type = unit;
-     max_use= None;
-     timeout= None;
-     kind = `Attached
-       {prefix = "";
-        subpath = [""];
-        fullpath = (Eliom_sessions.get_site_dir sp) @ [""];
-        get_name = Eliom_common.Att_no;
-        post_name = Eliom_common.Att_no;
-        att_kind = `Internal (`Service, `Get);
-      };
-     https = https;
-   }
-
-let static_dir ~sp = static_dir_ ~sp ()
-
-let https_static_dir ~sp = static_dir_ ~https:true ~sp ()
-
-let get_static_dir_ ?(https = false) ~sp ~get_params () =
-    {
-     pre_applied_parameters = [];
-     get_params_type = 
-        suffix_prod 
-          (all_suffix Eliom_common.eliom_suffix_name)
-          get_params;
-     post_params_type = unit;
-     max_use= None;
-     timeout= None;
-     kind = `Attached
-       {prefix = "";
-        subpath = [""];
-        fullpath = (Eliom_sessions.get_site_dir sp) @ [""];
-        get_name = Eliom_common.Att_no;
-        post_name = Eliom_common.Att_no;
-        att_kind = `Internal (`Service, `Get);
-      };
-     https = https;
-   }
-
-let static_dir_with_params ~sp ~get_params = 
-  get_static_dir_ ~sp ~get_params ()
-
-let https_static_dir_with_params ~sp ~get_params = 
-  get_static_dir_ ~https:true ~sp ~get_params ()
-
+let uniqueid =
+  let r = ref (-1) in
+  fun () -> r := !r + 1; !r
 
 (****************************************************************************)
 (****************************************************************************)
@@ -243,14 +106,16 @@ let https_static_dir_with_params ~sp ~get_params =
 let new_service_aux_aux
     ~https
     ~prefix
-    ~(path : url_path)
+    ~(path : Ocsigen_lib.url_path)
     ~site_dir
     ~kind
+    ?(redirect_suffix = true)
+    ?(keep_nl_params = `None)
     ~get_params
     ~post_params =
 (* ici faire une vrification "duplicate parameter" ? *)
   {
-   pre_applied_parameters = [];
+   pre_applied_parameters = Ocsigen_lib.String_Table.empty, [];
    get_params_type = get_params;
    post_params_type = post_params;
    max_use= None;
@@ -260,16 +125,20 @@ let new_service_aux_aux
       subpath = path;
       fullpath = site_dir @ path;
       att_kind = kind;
-      get_name = Eliom_common.Att_no;
-      post_name = Eliom_common.Att_no;
+      get_name = Eliom_common.SAtt_no;
+      post_name = Eliom_common.SAtt_no;
+      redirect_suffix = redirect_suffix
     };
    https = https;
+   keep_nl_params = keep_nl_params;
  }
 
 let new_service_aux
     ?sp
     ~https
     ~path
+    ?redirect_suffix
+    ?keep_nl_params
     ~get_params =
   match sp with
   | None ->
@@ -287,6 +156,8 @@ let new_service_aux
             ~path
             ~site_dir: sitedata.Eliom_common.site_dir
             ~kind:(`Internal (`Service, `Get))
+            ?redirect_suffix
+            ?keep_nl_params
             ~get_params
             ~post_params:unit
           in
@@ -307,6 +178,8 @@ let new_service_aux
         ~path:path
         ~site_dir:(Eliom_sessions.get_site_dir sp)
         ~kind:(`Internal (`Service, `Get))
+        ?redirect_suffix
+        ?keep_nl_params
         ~get_params
         ~post_params:unit
 
@@ -314,6 +187,7 @@ let new_service_aux
 let new_external_service
     ~prefix
     ~path
+    ?keep_nl_params
     ~get_params
     ~post_params
     () =
@@ -322,11 +196,13 @@ let new_external_service
     ~https:false (* not used for external links *)
     ~prefix
     ~path:(remove_internal_slash
-            (if suffix
-            then path@[Eliom_common.eliom_suffix_internal_name]
-            else path))
+            (match suffix with
+               | None -> path
+               | _ -> path@[Eliom_common.eliom_suffix_internal_name]))
     ~site_dir:[]
     ~kind:`External
+    ?keep_nl_params
+    ~redirect_suffix:false
     ~get_params
     ~post_params
 
@@ -334,25 +210,30 @@ let new_service
     ?sp
     ?(https = false)
     ~path
+    ?keep_nl_params
     ~get_params
     () =
   let suffix = contains_suffix get_params in
   new_service_aux
     ?sp
     ~https
-    ~path:(if suffix
-    then path@[Eliom_common.eliom_suffix_internal_name]
-    else path)
+    ~path:(match suffix with
+             | None -> path
+             | _ -> path@[Eliom_common.eliom_suffix_internal_name])
+    ?keep_nl_params
+    ?redirect_suffix:suffix
     ~get_params
 
-let new_naservice_num () = new_state ()
-
 let new_coservice
     ?name
+    ?(csrf_safe = false)
+    ?csrf_session_name
+    ?csrf_secure_session
     ?max_use
     ?timeout
     ?(https = false)
     ~fallback
+    ?keep_nl_params
     ~get_params
     () =
   let `Attached k = fallback.kind in
@@ -365,19 +246,36 @@ let new_coservice
    get_params_type = add_pref_params Eliom_common.co_param_prefix get_params;
    kind = `Attached
      {k with
-      get_name = 
-         (match name with
-           | None -> Eliom_common.Att_anon (new_state ())
-           | Some name -> Eliom_common.Att_named name);
+      get_name =
+         (if csrf_safe
+          then Eliom_common.SAtt_csrf_safe (uniqueid (),
+                                            csrf_session_name,
+                                            csrf_secure_session)
+          else
+            (match name with
+               | None -> Eliom_common.SAtt_anon (new_state ())
+               | Some name -> Eliom_common.SAtt_named name));
         att_kind = `Internal (`Coservice, `Get);
      };
-   https = https || fallback.https
+   https = https || fallback.https;
+   keep_nl_params = match keep_nl_params with 
+     | None -> fallback.keep_nl_params | Some k -> k;
  }
 (* Warning: here no GET parameters for the fallback.
    Preapply services if you want fallbacks with GET parameters *)
 
 
-let new_coservice' ?name ?max_use ?timeout ?(https = false) ~get_params () =
+let new_coservice' 
+    ?name 
+    ?(csrf_safe = false)
+    ?csrf_session_name
+    ?csrf_secure_session
+    ?max_use
+    ?timeout
+    ?(https = false)
+    ?(keep_nl_params = `Persistent)
+    ~get_params
+    () =
   (* (match Eliom_common.global_register_allowed () with
   | Some _ -> Eliom_common.add_unregistered_na n;
   | _ -> () (* Do we accept unregistered non-attached coservices? *)); *)
@@ -390,27 +288,32 @@ let new_coservice' ?name ?max_use ?timeout ?(https = false) ~get_params () =
 (*VVV allow timeout and max_use for named coservices? *)
           max_use= max_use;
           timeout= timeout;
-          pre_applied_parameters = [];
+          pre_applied_parameters = Ocsigen_lib.String_Table.empty, [];
           get_params_type = 
             add_pref_params Eliom_common.na_co_param_prefix get_params;
           post_params_type = unit;
           kind = `Nonattached
             {na_name = 
-                match name with
-                  | None ->
-                      Eliom_common.Na_get' (new_naservice_num ())
-                  | Some name -> Eliom_common.Na_get_ name;
-            ;
+                (if csrf_safe
+                 then Eliom_common.SNa_get_csrf_safe (uniqueid (),
+                                                      csrf_session_name,
+                                                      csrf_secure_session)
+                 else
+                   match name with
+                     | None -> Eliom_common.SNa_get' (new_state ())
+                     | Some name -> Eliom_common.SNa_get_ name);
              na_kind = `Get;
             };
           https = https;
+          keep_nl_params = keep_nl_params;
         }
 
 
 (****************************************************************************)
-(** Register a service with post parameters in the server *)
-let new_post_service_aux ~sp ~https ~fallback ~post_params =
-(** Create a main service (not a coservice) internal, post only *)
+(* Register a service with post parameters in the server *)
+let new_post_service_aux ~sp ~https ~fallback 
+    ?(keep_nl_params = `None) ~post_params =
+(* Create a main service (not a coservice) internal, post only *)
 (* ici faire une vrification "duplicate parameter" ? *)
   let `Attached k1 = fallback.kind in
   let `Internal (k, _) = k1.att_kind in
@@ -426,12 +329,15 @@ let new_post_service_aux ~sp ~https ~fallback ~post_params =
       fullpath = k1.fullpath;
       att_kind = `Internal (k, `Post);
       get_name = k1.get_name;
-      post_name = Eliom_common.Att_no;
+      post_name = Eliom_common.SAtt_no;
+      redirect_suffix = false;
     };
    https = https;
+   keep_nl_params = keep_nl_params;
  }
 
-let new_post_service ?sp ?(https = false) ~fallback ~post_params () =
+let new_post_service ?sp ?(https = false) ~fallback 
+    ?keep_nl_params ~post_params () =
   (* (if post_params = TUnit
   then Ocsigen_messages.warning "Probably error in the module: \
       Creation of a POST service without POST parameters.");
@@ -442,7 +348,9 @@ let new_post_service ?sp ?(https = false) ~fallback ~post_params () =
   let `Attached k1 = fallback.kind in
   let `Internal (kind, _) = k1.att_kind in
   let path = k1.subpath in
-  let u = new_post_service_aux ~sp ~https ~fallback ~post_params in
+  let u = new_post_service_aux ~sp ~https ~fallback 
+    ?keep_nl_params ~post_params 
+  in
   match sp with
   | None ->
       (match Eliom_common.global_register_allowed () with
@@ -461,7 +369,17 @@ let new_post_service ?sp ?(https = false) ~fallback ~post_params () =
 
 
 let new_post_coservice 
-    ?name ?max_use ?timeout ?(https = false) ~fallback ~post_params () =
+    ?name
+    ?(csrf_safe = false)
+    ?csrf_session_name
+    ?csrf_secure_session
+    ?max_use
+    ?timeout
+    ?(https = false)
+    ~fallback
+    ?keep_nl_params
+    ~post_params
+    () =
   let `Attached k1 = fallback.kind in
   (* (match Eliom_common.global_register_allowed () with
   | Some _ -> Eliom_common.add_unregistered k1.path;
@@ -474,11 +392,18 @@ let new_post_coservice
      {k1 with
       att_kind = `Internal (`Coservice, `Post);
       post_name = 
-         (match name with
-            | None -> Eliom_common.Att_anon (new_state ())
-            | Some name -> Eliom_common.Att_named name);
-    };
+         (if csrf_safe
+          then Eliom_common.SAtt_csrf_safe (uniqueid (),
+                                            csrf_session_name,
+                                            csrf_secure_session)
+          else
+            (match name with
+               | None -> Eliom_common.SAtt_anon (new_state ())
+               | Some name -> Eliom_common.SAtt_named name));
+     };
    https = https;
+   keep_nl_params = match keep_nl_params with 
+     | None -> fallback.keep_nl_params | Some k -> k;
  }
 (* It is not possible to make a new_post_coservice function
    with an optional ?fallback parameter
@@ -487,10 +412,15 @@ let new_post_coservice
  *)
 
 
-(*VVV Warning: keep_get_na_params is experimental *)
 let new_post_coservice'
     ?name
-    ?max_use ?timeout ?(keep_get_na_params = true) ?(https = false)
+    ?(csrf_safe = false)
+    ?csrf_session_name
+    ?csrf_secure_session
+    ?max_use ?timeout
+    ?(https = false)
+    ?(keep_nl_params = `All)
+    ?(keep_get_na_params = true)
     ~post_params () =
   (* match Eliom_common.global_register_allowed () with
   | Some _ -> Eliom_common.add_unregistered None
@@ -499,18 +429,24 @@ let new_post_coservice'
 (*VVV allow timeout and max_use for named coservices? *)
     max_use= max_use;
     timeout= timeout;
-    pre_applied_parameters = [];
+    pre_applied_parameters = Ocsigen_lib.String_Table.empty, [];
     get_params_type = unit;
     post_params_type = post_params;
     kind = `Nonattached
       {na_name = 
-          (match name with
-            | None ->
-                Eliom_common.Na_post' (new_naservice_num ())
-            | Some name -> Eliom_common.Na_post_ name);
+          (if csrf_safe
+           then Eliom_common.SNa_post_csrf_safe (uniqueid (),
+                                                 csrf_session_name,
+                                                 csrf_secure_session)
+           else
+             (match name with
+                | None ->
+                    Eliom_common.SNa_post' (new_state ())
+                | Some name -> Eliom_common.SNa_post_ name));
        na_kind = `Post keep_get_na_params;
       };
     https = https;
+    keep_nl_params = keep_nl_params;
   }
 
 
@@ -532,7 +468,7 @@ let new_get_post_coservice'
    max_use= max_use;
    timeout= timeout;
    kind = `Nonattached
-   {na_name = (fst fallback.na_name, Some (new_naservice_num ()));
+   {na_name = (fst fallback.na_name, Some (new_state ()));
    na_kind = `Internal (`NonAttachedCoservice, `Post);
    }
   https = https;
@@ -547,71 +483,7 @@ Forms towards that kind of service are not implemented
 *)
 
 
-let rec append_suffix l m = match l with
-  | [] -> m
-  | [eliom_suffix_internal_name] -> m
-  | a::ll -> a::(append_suffix ll m)
-
-let preapply ~service getparams =
-  let suff, params = construct_params_list service.get_params_type getparams in
-  {service with
-   pre_applied_parameters = params@service.pre_applied_parameters;
-   get_params_type = unit;
-   kind = match service.kind with
-   | `Attached k -> `Attached {k with
-                               subpath = (match suff with
-                               | Some suff -> append_suffix k.subpath suff
-                               | _ -> k.subpath);
-                               fullpath = (match suff with
-                               | Some suff -> append_suffix k.fullpath suff
-                               | _ -> k.fullpath);
-                             }
-   | k -> k
- }
-
-
-let void_coservice' =
-  {
-    max_use= None;
-    timeout= None;
-    pre_applied_parameters = [];
-    get_params_type = unit;
-    post_params_type = unit;
-    kind = `Nonattached
-      {na_name = Eliom_common.Na_void_dontkeep;
-       na_kind = `Get;
-      };
-    https = false;
-  }
-
-let https_void_coservice' =
-  {
-    max_use= None;
-    timeout= None;
-    pre_applied_parameters = [];
-    get_params_type = unit;
-    post_params_type = unit;
-    kind = `Nonattached
-      {na_name = Eliom_common.Na_void_dontkeep;
-       na_kind = `Get;
-      };
-    https = true;
-  }
-
-let void_hidden_coservice' = {void_coservice' with 
-                         kind = `Nonattached
-    {na_name = Eliom_common.Na_void_keep;
-     na_kind = `Get;
-    };
-                      }
-
-let https_void_hidden_coservice' = {void_coservice' with 
-                         kind = `Nonattached
-    {na_name = Eliom_common.Na_void_keep;
-     na_kind = `Get;
-    };
-                      }
-
+  
 
 (*****************************************************************************)
 let set_exn_handler ?sp h =
@@ -621,4 +493,105 @@ let set_exn_handler ?sp h =
 let add_service = Eliommod_services.add_service
 let add_naservice = Eliommod_naservices.add_naservice
 
-let erts_of_rst = Ocsigen_lib.id
+let eccookiel_of_escookiel = Ocsigen_lib.id
+let escookiel_of_eccookiel = Ocsigen_lib.id
+
+
+(*****************************************************************************)
+exception Unregistered_CSRF_safe_coservice
+
+let register_delayed_get_or_na_coservice ~sp (k, session_name, secure) =
+  let f =
+    try
+      let table = !(Eliom_sessions.get_session_service_table_if_exists
+                      ?session_name ?secure ~sp ())
+      in
+      Ocsigen_lib.Int_Table.find 
+        k table.Eliom_common.csrf_get_or_na_registration_functions
+    with Not_found ->
+      let table = Eliom_sessions.get_global_table ~sp in
+      try
+        Ocsigen_lib.Int_Table.find
+          k table.Eliom_common.csrf_get_or_na_registration_functions
+      with Not_found -> raise Unregistered_CSRF_safe_coservice
+  in
+  f ~sp:(Eliom_sessions.esp_of_sp sp)
+
+
+let register_delayed_post_coservice ~sp (k, session_name, secure) getname =
+  let f =
+    try
+      let table = !(Eliom_sessions.get_session_service_table_if_exists
+                      ?session_name ?secure ~sp ())
+      in
+      Ocsigen_lib.Int_Table.find 
+        k table.Eliom_common.csrf_post_registration_functions
+    with Not_found ->
+      let table = Eliom_sessions.get_global_table ~sp in
+      try
+        Ocsigen_lib.Int_Table.find
+          k table.Eliom_common.csrf_post_registration_functions
+      with Not_found -> raise Unregistered_CSRF_safe_coservice
+  in
+  f ~sp:(Eliom_sessions.esp_of_sp sp) getname
+
+
+let set_delayed_get_or_na_registration_function tables k f =
+  tables.Eliom_common.csrf_get_or_na_registration_functions <-
+    Ocsigen_lib.Int_Table.add
+      k
+      f
+      tables.Eliom_common.csrf_get_or_na_registration_functions
+
+let set_delayed_post_registration_function tables k f =
+  tables.Eliom_common.csrf_post_registration_functions <-
+    Ocsigen_lib.Int_Table.add
+    k
+    f
+    tables.Eliom_common.csrf_post_registration_functions
+
+
+(*****************************************************************************)
+let remove_service table service =
+  match get_kind_ service with
+    | `Attached attser ->
+        let key_kind = get_or_post (get_att_kind_ attser) in
+        let attserget = get_get_name_ attser in
+        let attserpost = get_post_name_ attser in
+        let sgpt = get_get_params_type_ service in
+        let sppt = get_post_params_type_ service in
+        Eliommod_services.remove_service table 
+          (get_sub_path_ attser)
+          {Eliom_common.key_state = (attserget, attserpost);
+           Eliom_common.key_kind = key_kind}
+          (if attserget = Eliom_common.SAtt_no
+             || attserpost = Eliom_common.SAtt_no
+           then (anonymise_params_type sgpt,
+                 anonymise_params_type sppt)
+           else (0, 0))
+    | `Nonattached naser ->
+        let na_name = get_na_name_ naser in
+        Eliommod_naservices.remove_naservice table na_name
+
+
+let unregister ?sp service =
+  let table = 
+    match sp with
+      | None ->
+          (match Eliom_common.global_register_allowed () with
+             | Some get_current_sitedata ->
+                 let sitedata = get_current_sitedata () in
+                 sitedata.Eliom_common.global_services
+           | _ -> raise
+               (Eliom_common.Eliom_function_forbidden_outside_site_loading
+                  "unregister"))
+      | Some sp -> get_global_table sp
+  in
+  remove_service table service
+
+
+let unregister_for_session ~sp ?session_name ?secure service =
+  let table =
+    !(Eliom_sessions.get_session_service_table ?secure ?session_name ~sp ())
+  in
+  remove_service table service
