@@ -26,7 +26,7 @@
 open Ocsigen_extensions
 open Eliom_parameters
 
-
+exception Wrong_session_table_for_CSRF_safe_coservice
 
 (** This function may be used for services that cannot be interrupted
   (no cooperation point for threads). It is defined by
@@ -40,8 +40,8 @@ val sync : ('a -> 'b -> 'c -> 'd) -> 'a -> 'b -> 'c -> 'd Lwt.t
     The strings are names and values.
  *)
 type cookie =
-  | Set of url_path option * float option * string * string * bool
-  | Unset of url_path option * string
+  | Set of Ocsigen_lib.url_path option * float option * string * string * bool
+  | Unset of Ocsigen_lib.url_path option * string
 
 (** Conversion fonction from Eliom cookies to server cookies.
     If [?oldtable] is present, cookies are added to this table
@@ -51,12 +51,6 @@ val cookie_table_of_eliom_cookies :
       sp:Eliom_sessions.server_params -> cookie list -> Ocsigen_http_frame.cookieset
 
 
-(** The type to send if you want to create your own modules for generating
-   pages
- *)
-type result_to_send =
-  | EliomResult of Ocsigen_http_frame.result
-  | EliomExn of (exn list * cookie list)
 
 
 
@@ -162,7 +156,8 @@ val register_eliom_module : string -> (unit -> unit) -> unit
 val new_service :
   ?sp: Eliom_sessions.server_params ->
   ?https:bool ->
-  path:url_path ->
+  path:Ocsigen_lib.url_path ->
+  ?keep_nl_params:[ `All | `Persistent | `None ] ->
   get_params:('get, [< suff ] as 'tipo,'gn) params_type ->
   unit ->
   ('get,unit,
@@ -183,7 +178,8 @@ val new_service :
 
 val new_external_service :
   prefix: string ->
-  path:url_path ->
+  path:Ocsigen_lib.url_path ->
+  ?keep_nl_params:[ `All | `Persistent | `None ] ->
   get_params:('get, [< suff ] as 'tipo, 'gn) params_type ->
   post_params:('post, [ `WithoutSuffix ], 'pn) params_type ->
   unit ->
@@ -212,6 +208,7 @@ val new_post_service :
                               ([ `Service | `Coservice ] as 'kind) * [`Get]] a_s ],
              [< suff] as 'tipo, 'gn, unit,
              [< `Registrable ]) service ->
+  ?keep_nl_params:[ `All | `Persistent | `None ] ->
   post_params: ('post, [`WithoutSuffix], 'pn) params_type ->
   unit ->
   ('get, 'post, [> `Attached of
@@ -231,6 +228,9 @@ val new_post_service :
 
 val new_coservice :
   ?name: string ->
+  ?csrf_safe: bool ->
+  ?csrf_session_name: string ->
+  ?csrf_secure_session: bool ->
   ?max_use:int ->
   ?timeout:float ->
   ?https:bool ->
@@ -238,6 +238,7 @@ val new_coservice :
     (unit, unit, [ `Attached of [ `Internal of [ `Service ] * [`Get]] a_s ],
      [ `WithoutSuffix ] as 'tipo,
      unit, unit, [< registrable ]) service ->
+  ?keep_nl_params:[ `All | `Persistent | `None ] ->
   get_params:
     ('get,[`WithoutSuffix],'gn) params_type ->
   unit ->
@@ -255,10 +256,41 @@ val new_coservice :
    Coservices can be named if the [?name] optional parameter
    is present or anonymous (in that case, a coservice number will be
    generated).
+
+   The [~timeout] parameter specifies a timeout (in seconds)
+   after which the coservice will disappear. This amount of time is
+   computed from the creation or the last call to the service.
+
+   The [~max_use] parameter specifies that the service can be used only
+   a fixed number of times.
+
+   If [~csrf_safe] is [true],
+   it will create a "CSRF-safe" service (the default is [false]). 
+   (In that case [~name] is ignored).
+   It means that the registration of the service will not actually
+   take place when [register] is called, but delayed and performed
+   each time a form is created. This allows to protect against CSRF attacks,
+   and should be use with a short timeout (and max_use).
+   (And you should probably use POST coservices in that case).
+   In that case, you can register the CSRF safe service either in the global
+   service table or in the session service table. But the actual registration,
+   that will occure when creating a link or a form, will always take
+   place in a session service table. This table is specified by the
+   [~csrf_session_name] and [~csrf_secure_session] parameters
+   (that correspond to [~session_name] and [~secure] for the delayed 
+    registration); it is default session table if they are absent.
+    Parameters [?session_name] and [?secure] of [register_for_session]
+    must have the same values as the one declared while creating the
+    CSRF safe coservice, otherwise the registration will fail
+    with {Eliom_services.Wrong_session_table_for_CSRF_safe_coservice}.
+   
  *)
 
 val new_post_coservice :
   ?name: string ->
+  ?csrf_safe: bool ->
+  ?csrf_session_name: string ->
+  ?csrf_secure_session: bool ->
   ?max_use:int ->
   ?timeout:float ->
   ?https:bool ->
@@ -266,6 +298,7 @@ val new_post_coservice :
                              [`Internal of [<`Service | `Coservice] * [`Get]] a_s ],
              [< suff ] as 'tipo,
              'gn, unit, [< `Registrable ]) service ->
+  ?keep_nl_params:[ `All | `Persistent | `None ] ->
   post_params: ('post, [`WithoutSuffix], 'pn) params_type ->
   unit ->
   ('get, 'post,
@@ -278,9 +311,13 @@ val new_post_coservice :
 
 val new_coservice' :
   ?name:string ->
+  ?csrf_safe: bool ->
+  ?csrf_session_name: string ->
+  ?csrf_secure_session: bool ->
   ?max_use:int ->
   ?timeout:float ->
   ?https:bool ->
+  ?keep_nl_params:[ `All | `Persistent | `None ] ->
   get_params:
     ('get, [`WithoutSuffix], 'gn) params_type ->
   unit ->
@@ -298,10 +335,14 @@ val new_coservice' :
 
 val new_post_coservice' :
   ?name:string ->
+  ?csrf_safe: bool ->
+  ?csrf_session_name: string ->
+  ?csrf_secure_session: bool ->
   ?max_use:int ->
   ?timeout:float ->
-  ?keep_get_na_params:bool ->
   ?https:bool ->
+  ?keep_nl_params:[ `All | `Persistent | `None ] ->
+  ?keep_get_na_params:bool ->
   post_params: ('post, [`WithoutSuffix], 'pn) params_type ->
   unit ->
   (unit, 'post,
@@ -332,7 +373,7 @@ val new_get_post_coservice' :
 *)
 
 
-(** {2 Misc} *)
+(** {2 Predefined services} *)
 
 val static_dir :
   sp:Eliom_sessions.server_params ->
@@ -359,7 +400,9 @@ val https_static_dir :
 
 val static_dir_with_params :
   sp:Eliom_sessions.server_params ->
+  ?keep_nl_params:[ `All | `Persistent | `None ] ->
   get_params:('a, [`WithoutSuffix], 'an) params_type ->
+  unit ->
   ((string list * 'a), unit,
    [> `Attached of
       [> `Internal of [> `Service ] * [> `Get] ] a_s ],
@@ -370,7 +413,9 @@ val static_dir_with_params :
 
 val https_static_dir_with_params :
   sp:Eliom_sessions.server_params ->
+  ?keep_nl_params:[ `All | `Persistent | `None ] ->
   get_params:('a, [`WithoutSuffix], 'an) params_type ->
+  unit ->
   ((string list * 'a), unit,
    [> `Attached of
       [> `Internal of [> `Service ] * [> `Get] ] a_s ],
@@ -420,6 +465,8 @@ val https_void_hidden_coservice' :
 (** The same, but forcing https. *)
 
 
+(** {2 Misc} *)
+
 val preapply :
     service:('a, 'b, [> `Attached of 'd a_s ] as 'c,
      [< suff ], 'e, 'f, 'g)
@@ -432,11 +479,40 @@ val preapply :
    Preapplied services may be used in links or as fallbacks for coservices
  *)
 
+val add_non_localized_get_parameters :
+  params:('p, [ `WithoutSuffix ], 'pn) non_localized_params ->
+  service:('a, 'b, 'c, 'd, 'e, 'f, 'g) service ->
+  ('a * 'p, 'b, 'c, 'd, 'e * 'pn, 'f, 'g) service
+(** Adds non localized GET parameters to a service *)
+
+val add_non_localized_post_parameters :
+  params:('p, [ `WithoutSuffix ], 'pn) non_localized_params ->
+  service:('a, 'b, 'c, 'd, 'e, 'f, 'g) service ->
+  ('a, 'b * 'p, 'c, 'd, 'e, 'f * 'pn, 'g) service
+(** Adds non localized POST parameters to a service *)
+
+
+val unregister :
+  ?sp:Eliom_sessions.server_params ->
+  ('a, 'b, [< `Attached of [> `Internal of 'c * [> `Get ] ] a_s
+   | `Nonattached of 'd na_s ], 'e, 'f, 'g, 'h) service ->
+  unit
+(** Unregister a service from global table *)
+
+val unregister_for_session :
+  sp:Eliom_sessions.server_params ->
+  ?session_name:string ->
+  ?secure:bool ->
+  ('a, 'b, [< `Attached of [> `Internal of 'c * [> `Get ] ] a_s
+   | `Nonattached of 'd na_s ], 'e, 'f, 'g, 'h) service ->
+  unit
+(** Unregister a service from session table *)
+
 
 (** {2 Using your own error pages} *)
 
 
-(** allows to use your own error pages
+(** Allows to use your own error pages
    (404, or any exception during page generation).
 
     {e Warning: If you use this function after the initialisation phase,
@@ -444,8 +520,9 @@ val preapply :
     exception {!Eliom_common.Eliom_function_forbidden_outside_site_loading}.}
  *)
 val set_exn_handler :
-    ?sp:Eliom_sessions.server_params ->
-      (Eliom_sessions.server_params -> exn -> result_to_send Lwt.t) -> unit
+  ?sp:Eliom_sessions.server_params ->
+  (Eliom_sessions.server_params -> exn -> Ocsigen_http_frame.result Lwt.t) ->
+  unit
 
 
 
@@ -455,23 +532,62 @@ val set_exn_handler :
 (**/**)
 val get_kind_ : ('a, 'b, 'c, 'd, 'e, 'f, 'g) service -> 'c
 val get_pre_applied_parameters_ : ('a, 'b, 'c, 'd, 'e, 'f, 'g) service ->
+  (string * string) list Ocsigen_lib.String_Table.t *
   (string * string) list
 val get_get_params_type_ : ('a, 'b, 'c, 'd, 'e, 'f, 'g) service ->
   ('a, 'd, 'e) Eliom_parameters.params_type
 val get_post_params_type_ : ('a, 'b, 'c, 'd, 'e, 'f, 'g) service ->
   ('b, [ `WithoutSuffix ], 'f) Eliom_parameters.params_type
 val get_att_kind_ : 'a a_s -> 'a
-val get_sub_path_ : 'a a_s -> url_path
-val get_full_path_ : 'a a_s -> url_path
+val get_sub_path_ : 'a a_s -> Ocsigen_lib.url_path
+val get_full_path_ : 'a a_s -> Ocsigen_lib.url_path
 val get_prefix_ : 'a a_s -> string
-val get_get_name_ : 'a a_s -> Eliom_common.att_key
-val get_post_name_ : 'a a_s -> Eliom_common.att_key
-val get_na_name_ : 'a na_s -> Eliom_common.na_key
+val get_get_name_ : 'a a_s -> Eliom_common.att_key_serv
+val get_post_name_ : 'a a_s -> Eliom_common.att_key_serv
+val get_redirect_suffix_ : 'a a_s -> bool
+val get_na_name_ : 'a na_s -> Eliom_common.na_key_serv
 val get_na_kind_ : 'a na_s -> [ `Get | `Post of bool ]
 val get_max_use_ : ('a, 'b, 'c, 'd, 'e, 'f, 'g) service -> int option
 val get_timeout_ : ('a, 'b, 'c, 'd, 'e, 'f, 'g) service -> float option
 val get_https : ('a, 'b, 'c, 'd, 'e, 'f, 'g) service -> bool
-(* val reconstruct_absolute_url_path : url_path -> url_path -> url_path option -> string
-val reconstruct_relative_url_path : url_path -> url_path -> url_path option -> string
+(* val reconstruct_absolute_url_path : Ocsigen_lib.url_path -> Ocsigen_lib.url_path -> Ocsigen_lib.url_path option -> string
+val reconstruct_relative_url_path : Ocsigen_lib.url_path -> Ocsigen_lib.url_path -> Ocsigen_lib.url_path option -> string
 *)
-val erts_of_rst : result_to_send -> Eliom_common.result_to_send
+val escookiel_of_eccookiel : Eliom_common.cookie list -> cookie list
+val eccookiel_of_escookiel : cookie list -> Eliom_common.cookie list
+val keep_nl_params : ('a, 'b, 'c, 'd, 'e, 'f, 'g) service -> 
+  [ `All | `Persistent | `None ]
+
+val get_or_post : [> `Internal of 'a * [> `Get ] ] ->
+  Ocsigen_http_frame.Http_header.http_method
+
+val change_get_num :
+  ('a, 'b, 'c, 'd, 'e, 'f, 'g) service ->
+  'h a_s ->
+  Eliom_common.att_key_serv ->
+  ('a, 'b, [> `Attached of 'h a_s ], 'd, 'e, 'f, 'i) service
+
+
+val new_state : unit -> string
+
+(*****************************************************************************)
+
+val register_delayed_get_or_na_coservice :
+  sp:Eliom_sessions.server_params ->
+  (int * string option * bool option) -> string
+
+val register_delayed_post_coservice :
+  sp:Eliom_sessions.server_params -> 
+  (int * string option * bool option) -> Eliom_common.att_key_serv -> string
+
+val set_delayed_get_or_na_registration_function :
+  Eliom_common.tables ->
+  int -> 
+  (sp:Eliom_common.server_params -> string) -> unit
+
+val set_delayed_post_registration_function :
+  Eliom_common.tables ->
+  int -> 
+  (sp:Eliom_common.server_params -> Eliom_common.att_key_serv -> string) -> 
+  unit
+
