@@ -64,7 +64,8 @@ let get_initial_get_params ~sp =
 let get_get_params_string ~sp =
   sp.Eliom_common.sp_request.request_info.ri_get_params_string
 let get_post_params ~sp =
-  Lazy.force sp.Eliom_common.sp_request.request_info.ri_post_params
+  sp.Eliom_common.sp_request.request_info.ri_post_params
+    sp.Eliom_common.sp_request.request_config
 let get_all_post_params ~sp =
   sp.Eliom_common.sp_si.Eliom_common.si_all_post_params
 let get_original_full_path_string ~sp =
@@ -95,12 +96,33 @@ let get_ssl ~sp =
   sp.Eliom_common.sp_request.request_info.ri_ssl
 let get_other_get_params ~sp =
   sp.Eliom_common.sp_si.Eliom_common.si_other_get_params
+let get_nl_get_params ~sp =
+  sp.Eliom_common.sp_si.Eliom_common.si_nl_get_params
+let get_persistent_nl_get_params ~sp =
+  Lazy.force sp.Eliom_common.sp_si.Eliom_common.si_persistent_nl_get_params
+let get_nl_post_params ~sp =
+  sp.Eliom_common.sp_si.Eliom_common.si_nl_post_params
 let get_suffix ~sp =
   sp.Eliom_common.sp_suffix
 let get_session_name ~sp =
   sp.Eliom_common.sp_fullsessname
-let get_exn ~sp =
-  sp.Eliom_common.sp_request.request_info.ri_extension_info
+let get_request_cache ~sp =
+  sp.Eliom_common.sp_request.request_info.ri_request_cache
+let clean_request_cache ~sp =
+  sp.Eliom_common.sp_request.request_info.ri_request_cache <- 
+    Polytables.create ()
+let get_link_too_old ~sp =
+  try
+    Polytables.get
+      ~table:sp.Eliom_common.sp_request.request_info.ri_request_cache
+      ~key:Eliom_common.eliom_link_too_old
+  with Not_found -> false
+let get_expired_service_sessions ~sp =
+  try
+    Polytables.get
+      ~table:sp.Eliom_common.sp_request.request_info.ri_request_cache
+      ~key:Eliom_common.eliom_service_session_expired
+  with Not_found -> []
 let get_config_default_charset ~sp =
   Ocsigen_charset_mime.default_charset
     sp.Eliom_common.sp_request.request_config.charset_assoc
@@ -112,18 +134,20 @@ let get_persistent_cookies ~sp =
   sp.Eliom_common.sp_si.Eliom_common.si_persistent_session_cookies
 let get_previous_extension_error_code ~sp =
   sp.Eliom_common.sp_si.Eliom_common.si_previous_extension_error
+let get_si ~sp =
+  sp.Eliom_common.sp_si
 
 let get_service_session_cookie ?session_name ?secure ~sp () =
   try
     let c = Eliommod_sersess.find_service_cookie_only ?session_name ~secure ~sp () in
     Some c.Eliom_common.sc_value
-  with Not_found -> None
+  with Not_found | Eliom_common.Eliom_Session_expired -> None
 
 let get_volatile_data_session_cookie ?session_name ?secure ~sp () =
   try
     let c = Eliommod_datasess.find_data_cookie_only ?session_name ~secure ~sp () in
     Some c.Eliom_common.dc_value
-  with Not_found -> None
+  with Not_found | Eliom_common.Eliom_Session_expired -> None
 
 let get_persistent_data_session_cookie ?session_name ?secure ~sp () =
   catch
@@ -132,8 +156,11 @@ let get_persistent_data_session_cookie ?session_name ?secure ~sp () =
         ?session_name ~secure ~sp () >>= fun c ->
       return (Some c.Eliom_common.pc_value)
     )
-    (function Not_found -> return None | e -> fail e)
+    (function
+       | Not_found | Eliom_common.Eliom_Session_expired -> return None
+       | e -> fail e)
 
+(*
 let get_default_service_session_timeout = Eliommod_timeouts.get_default_service_timeout
 let set_default_service_session_timeout = Eliommod_timeouts.set_default_service_timeout
 
@@ -146,25 +173,68 @@ let set_default_volatile_data_session_timeout =
 let set_default_volatile_session_timeout =
   Eliommod_timeouts.set_default_volatile_timeout
 
+let get_default_persistent_data_session_timeout =
+  Eliommod_timeouts.get_default_persistent_timeout
+
+let set_default_persistent_data_session_timeout =
+  Eliommod_timeouts.set_default_persistent_timeout
+  *)
+
 let set_global_service_session_timeout
-    ?session_name ?sp ?(recompute_expdates = false) timeout =
+    ?session_name ?sp ?(recompute_expdates = false)
+    ?(override_configfile = false) timeout =
   let sitedata = find_sitedata "set_global_service_timeout" sp in
-  Eliommod_timeouts.set_global_service_timeout
-    ~session_name ~recompute_expdates sitedata timeout
+  match session_name with
+    | Some session_name ->
+        Eliommod_timeouts.set_global_service_timeout
+          ~session_name ~recompute_expdates override_configfile sitedata timeout
+    | None ->
+        Eliommod_timeouts.set_default_global_service_timeout
+          override_configfile false sitedata timeout
 
 let set_global_volatile_data_session_timeout
-    ?session_name ?sp ?(recompute_expdates = false) timeout =
+    ?session_name ?sp ?(recompute_expdates = false)
+    ?(override_configfile = false) timeout =
   let sitedata = find_sitedata "set_global_data_timeout" sp in
-  Eliommod_timeouts.set_global_data_timeout
-    ~session_name ~recompute_expdates sitedata timeout
+  match session_name with
+    | Some session_name ->
+        Eliommod_timeouts.set_global_data_timeout
+          ~session_name ~recompute_expdates override_configfile sitedata timeout
+    | None ->
+        Eliommod_timeouts.set_default_global_data_timeout
+          override_configfile false sitedata timeout
 
 let set_global_volatile_session_timeout
-    ?session_name ?sp ?(recompute_expdates = false) timeout =
+    ?session_name ?sp ?(recompute_expdates = false) 
+    ?(override_configfile = false) timeout =
   let sitedata = find_sitedata "set_global_volatile_timeouts" sp in
-  Eliommod_timeouts.set_global_service_timeout
-    ~session_name ~recompute_expdates sitedata timeout;
-  Eliommod_timeouts.set_global_data_timeout
-    ~session_name ~recompute_expdates sitedata timeout
+  match session_name with
+    | Some session_name ->
+        Eliommod_timeouts.set_global_service_timeout
+          ~session_name ~recompute_expdates
+          override_configfile sitedata timeout;
+        Eliommod_timeouts.set_global_data_timeout
+          ~session_name ~recompute_expdates
+          override_configfile sitedata timeout
+    | None ->
+        Eliommod_timeouts.set_default_global_service_timeout
+          override_configfile false sitedata timeout;
+        Eliommod_timeouts.set_default_global_data_timeout
+          override_configfile false sitedata timeout
+
+let set_global_persistent_data_session_timeout
+    ?session_name ?sp ?(recompute_expdates = false)
+    ?(override_configfile = false) timeout =
+  let sitedata = find_sitedata "set_global_persistent_timeout" sp in
+  match session_name with
+    | Some session_name ->
+        Eliommod_timeouts.set_global_persistent_timeout
+          ~session_name ~recompute_expdates
+          override_configfile sitedata timeout
+    | None ->
+        Eliommod_timeouts.set_default_global_service_timeout
+          override_configfile false sitedata timeout
+
 
 let get_global_service_session_timeout ?session_name ?sp () =
   let sitedata = find_sitedata "get_global_timeout" sp in
@@ -174,22 +244,13 @@ let get_global_volatile_data_session_timeout ?session_name ?sp () =
   let sitedata = find_sitedata "get_global_timeout" sp in
   Eliommod_timeouts.get_global_data_timeout ?session_name sitedata
 
-let get_default_persistent_data_session_timeout =
-  Eliommod_timeouts.get_default_persistent_timeout
-let set_default_persistent_data_session_timeout =
-  Eliommod_timeouts.set_default_persistent_timeout
-
-let set_global_persistent_data_session_timeout
-    ?session_name ?sp ?(recompute_expdates = false) timeout =
-  let sitedata = find_sitedata "set_global_persistent_timeout" sp in
-  Eliommod_timeouts.set_global_persistent_timeout
-    ~session_name ~recompute_expdates sitedata timeout
-
 let get_global_persistent_data_session_timeout ?session_name ?sp () =
   let sitedata = find_sitedata "get_global_persistent_timeout" sp in
   Eliommod_timeouts.get_global_persistent_timeout ?session_name sitedata
 
 
+
+(* Now for current session *)
 let set_service_session_timeout ?session_name ?secure ~sp t =
   let c = 
     Eliommod_sersess.find_or_create_service_cookie ?session_name ~secure ~sp () 
@@ -216,7 +277,7 @@ let unset_service_session_timeout ?session_name ?secure ~sp () =
     in
     let tor = c.Eliom_common.sc_timeout in
     tor := Eliom_common.TGlobal
-  with Not_found -> ()
+  with Not_found | Eliom_common.Eliom_Session_expired -> ()
 
 let unset_volatile_data_session_timeout ?session_name ?secure ~sp () =
   try
@@ -225,7 +286,7 @@ let unset_volatile_data_session_timeout ?session_name ?secure ~sp () =
     in
     let tor = c.Eliom_common.dc_timeout in
     tor := Eliom_common.TGlobal
-  with Not_found -> ()
+  with Not_found | Eliom_common.Eliom_Session_expired -> ()
 
 
 let get_service_session_timeout ?session_name ?secure ~sp () =
@@ -240,7 +301,7 @@ let get_service_session_timeout ?session_name ?secure ~sp () =
           ?session_name sp.Eliom_common.sp_sitedata
     | Eliom_common.TNone -> None
     | Eliom_common.TSome t -> Some t
-  with Not_found ->
+  with Not_found | Eliom_common.Eliom_Session_expired ->
     Eliommod_timeouts.get_global_service_timeout
       ?session_name sp.Eliom_common.sp_sitedata
 
@@ -256,7 +317,7 @@ let get_volatile_data_session_timeout ?session_name ?secure ~sp () =
           ?session_name sp.Eliom_common.sp_sitedata
     | Eliom_common.TNone -> None
     | Eliom_common.TSome t -> Some t
-  with Not_found ->
+  with Not_found | Eliom_common.Eliom_Session_expired ->
     Eliommod_timeouts.get_global_data_timeout
       ?session_name sp.Eliom_common.sp_sitedata
 
@@ -292,7 +353,9 @@ let unset_persistent_data_session_timeout ?session_name ?secure ~sp () =
       tor := Eliom_common.TGlobal;
       return ()
     )
-    (function Not_found -> return () | e -> fail e)
+    (function
+       | Not_found | Eliom_common.Eliom_Session_expired -> return () 
+       | e -> fail e)
 
 let get_persistent_data_session_timeout ?session_name ?secure ~sp () =
   catch
@@ -309,7 +372,7 @@ let get_persistent_data_session_timeout ?session_name ?secure ~sp () =
         | Eliom_common.TSome t -> Some t)
     )
     (function
-      | Not_found ->
+      | Not_found | Eliom_common.Eliom_Session_expired ->
           return
             (Eliommod_timeouts.get_global_persistent_timeout
                ~session_name sp.Eliom_common.sp_sitedata)
@@ -324,30 +387,37 @@ type 'a session_data =
   | Data of 'a
 
 
-let set_service_session_group ?set_max ?session_name ?secure ~sp n =
-  let c = Eliommod_sersess.find_or_create_service_cookie
+let set_service_session_group ?set_max ?session_name ?secure ~sp session_group =
+  let c =
+    Eliommod_sersess.find_or_create_service_cookie
+      ~set_session_group:session_group
       ?session_name ~secure ~sp ()
   in
-  let n =
-    Eliommod_sessiongroups.make_full_group_name
-      sp.Eliom_common.sp_sitedata.Eliom_common.site_dir_string (Some n)
-  in
-  let grp = c.Eliom_common.sc_session_group in
-  List.iter
-    (Eliom_common.close_service_session2 sp.Eliom_common.sp_sitedata None)
-    (Eliommod_sessiongroups.Serv.move ?set_max
-       sp.Eliom_common.sp_sitedata.Eliom_common.max_service_sessions_per_group
-       c.Eliom_common.sc_value !grp n);
-  grp := n
+  match set_max with
+    | None -> ()
+    | Some m -> 
+        Eliommod_sessiongroups.Data.set_max
+          c.Eliom_common.sc_session_group_node m
 
-let unset_service_session_group ?session_name ?secure ~sp () =
+let unset_service_session_group ?set_max ?session_name ?secure ~sp () =
   try
     let c = 
       Eliommod_sersess.find_service_cookie_only ?session_name ~secure ~sp () 
     in
-    let grp = c.Eliom_common.sc_session_group in
-    Eliommod_sessiongroups.Serv.remove c.Eliom_common.sc_value !grp;
-    grp := None
+    let n =
+      Eliommod_sessiongroups.make_full_group_name
+        sp.Eliom_common.sp_request.Ocsigen_extensions.request_info
+        sp.Eliom_common.sp_sitedata.Eliom_common.site_dir_string
+        (Eliom_common.get_mask4 sp.Eliom_common.sp_sitedata)
+        (Eliom_common.get_mask6 sp.Eliom_common.sp_sitedata)
+        None
+    in
+    let node = Eliommod_sessiongroups.Serv.move ?set_max
+      sp.Eliom_common.sp_sitedata
+      c.Eliom_common.sc_session_group_node n
+    in
+    c.Eliom_common.sc_session_group_node <- node;
+    c.Eliom_common.sc_session_group := n
   with
     | Not_found
     | Eliom_common.Eliom_Session_expired -> ()
@@ -358,36 +428,44 @@ let get_service_session_group ?session_name ?secure ~sp () =
       Eliommod_sersess.find_service_cookie_only ?session_name ~secure ~sp () 
     in
     match !(c.Eliom_common.sc_session_group) with
-      | None -> No_data
-      | Some v -> Data (snd (Eliommod_sessiongroups.getsessgrp v))
+      | _, Ocsigen_lib.Right _ -> No_data
+      | _, Ocsigen_lib.Left v -> Data v
   with
     | Not_found -> No_data
     | Eliom_common.Eliom_Session_expired -> Data_session_expired
 
-let set_volatile_data_session_group ?set_max ?session_name ?secure ~sp n =
+let set_volatile_data_session_group
+    ?set_max ?session_name ?secure ~sp session_group =
   let c = 
-    Eliommod_datasess.find_or_create_data_cookie ?session_name ~secure ~sp () 
+    Eliommod_datasess.find_or_create_data_cookie
+      ~set_session_group:session_group
+      ?session_name ~secure ~sp () 
   in
-  let n =
-    Eliommod_sessiongroups.make_full_group_name
-      sp.Eliom_common.sp_sitedata.Eliom_common.site_dir_string (Some n)
-  in
-  let grp = c.Eliom_common.dc_session_group in
-  List.iter
-    (Eliommod_datasess.close_data_session2 sp.Eliom_common.sp_sitedata None)
-    (Eliommod_sessiongroups.Data.move ?set_max
-       sp.Eliom_common.sp_sitedata.Eliom_common.max_volatile_data_sessions_per_group
-       c.Eliom_common.dc_value !grp n);
-  grp := n
+  match set_max with
+    | None -> ()
+    | Some m -> 
+        Eliommod_sessiongroups.Data.set_max
+          c.Eliom_common.dc_session_group_node m
 
-let unset_volatile_data_session_group ?session_name ?secure ~sp () =
+let unset_volatile_data_session_group ?set_max ?session_name ?secure ~sp () =
   try
     let c = 
       Eliommod_datasess.find_data_cookie_only ?session_name ~secure ~sp () 
     in
-    let grp = c.Eliom_common.dc_session_group in
-    Eliommod_sessiongroups.Data.remove c.Eliom_common.dc_value !grp;
-    grp := None
+    let n =
+      Eliommod_sessiongroups.make_full_group_name
+        sp.Eliom_common.sp_request.Ocsigen_extensions.request_info
+        sp.Eliom_common.sp_sitedata.Eliom_common.site_dir_string 
+        (Eliom_common.get_mask4 sp.Eliom_common.sp_sitedata)
+        (Eliom_common.get_mask6 sp.Eliom_common.sp_sitedata)
+        None
+    in
+    let node = Eliommod_sessiongroups.Data.move ?set_max
+      sp.Eliom_common.sp_sitedata
+      c.Eliom_common.dc_session_group_node n
+    in
+    c.Eliom_common.dc_session_group_node <- node;
+    c.Eliom_common.dc_session_group := n
   with
     | Not_found
     | Eliom_common.Eliom_Session_expired -> ()
@@ -398,8 +476,8 @@ let get_volatile_data_session_group ?session_name ?secure ~sp () =
       Eliommod_datasess.find_data_cookie_only ?session_name ~secure ~sp () 
     in
     match !(c.Eliom_common.dc_session_group) with
-      | None -> No_data
-      | Some v -> Data (snd (Eliommod_sessiongroups.getsessgrp v))
+      | _, Ocsigen_lib.Right _ -> No_data
+      | _, Ocsigen_lib.Left v -> Data v
   with
     | Not_found -> No_data
     | Eliom_common.Eliom_Session_expired -> Data_session_expired
@@ -409,11 +487,12 @@ let set_persistent_data_session_group ?set_max ?session_name ?secure ~sp n =
     ?session_name ~secure ~sp () >>= fun c ->
   let n =
     Eliommod_sessiongroups.make_persistent_full_group_name
+      sp.Eliom_common.sp_request.Ocsigen_extensions.request_info
       sp.Eliom_common.sp_sitedata.Eliom_common.site_dir_string (Some n)
   in
   let grp = c.Eliom_common.pc_session_group in
   Eliommod_sessiongroups.Pers.move ?set_max
-    sp.Eliom_common.sp_sitedata.Eliom_common.max_persistent_data_sessions_per_group
+    (fst sp.Eliom_common.sp_sitedata.Eliom_common.max_persistent_data_sessions_per_group)
     c.Eliom_common.pc_value !grp n >>= fun l ->
   Lwt_util.iter
         (Eliommod_persess.close_persistent_session2 None) l >>= fun () ->
@@ -452,22 +531,132 @@ let get_persistent_data_session_group ?session_name ?secure ~sp () =
        | e -> fail e)
 
 
-let set_default_max_service_sessions_per_group ~sp n =
-  sp.Eliom_common.sp_sitedata.Eliom_common.max_service_sessions_per_group <- n
 
-let set_default_max_volatile_data_sessions_per_group ~sp n =
-  sp.Eliom_common.sp_sitedata.Eliom_common.max_volatile_data_sessions_per_group
-  <- n
 
-let set_default_max_persistent_data_sessions_per_group ~sp n =
-  sp.Eliom_common.sp_sitedata.Eliom_common.max_persistent_data_sessions_per_group <- n
+
+(* max *)
+let set_default_max_service_sessions_per_group
+    ?sp ?(override_configfile = false) n =
+  let sitedata = 
+    find_sitedata "set_default_max_service_sessions_per_group" sp 
+  in
+  let b = snd sitedata.Eliom_common.max_service_sessions_per_group in 
+  if override_configfile || not b
+  then 
+    sitedata.Eliom_common.max_service_sessions_per_group <- (n, b)
+
+let set_default_max_volatile_data_sessions_per_group
+    ?sp ?(override_configfile = false) n =
+  let sitedata = 
+    find_sitedata "set_default_max_service_sessions_per_group" sp 
+  in
+  let b = snd sitedata.Eliom_common.max_volatile_data_sessions_per_group in
+  if override_configfile || not b
+  then 
+    sitedata.Eliom_common.max_volatile_data_sessions_per_group <- (n, b)
+
+let set_default_max_persistent_data_sessions_per_group
+    ?sp ?(override_configfile = false) n =
+  let sitedata = 
+    find_sitedata "set_default_max_service_sessions_per_group" sp 
+  in
+  let b = snd sitedata.Eliom_common.max_persistent_data_sessions_per_group in
+  if override_configfile || not b
+  then 
+    sitedata.Eliom_common.max_persistent_data_sessions_per_group <- (n, b)
+
+let set_default_max_service_sessions_per_subnet
+    ?sp ?(override_configfile = false) n =
+  let sitedata = 
+    find_sitedata "set_default_max_service_sessions_per_group" sp 
+  in
+  let b = snd sitedata.Eliom_common.max_service_sessions_per_subnet in
+  if override_configfile || not b
+  then 
+    sitedata.Eliom_common.max_service_sessions_per_subnet <- (n, b)
+
+let set_default_max_volatile_data_sessions_per_subnet
+    ?sp ?(override_configfile = false) n =
+  let sitedata = 
+    find_sitedata "set_default_max_service_sessions_per_group" sp 
+  in
+  let b = snd sitedata.Eliom_common.max_volatile_data_sessions_per_subnet in
+  if override_configfile || not b
+  then 
+    sitedata.Eliom_common.max_volatile_data_sessions_per_subnet <- (n, b)
+
+let set_default_max_volatile_sessions_per_group ?sp ?override_configfile n =
+  set_default_max_service_sessions_per_group ?sp ?override_configfile n;
+  set_default_max_volatile_data_sessions_per_group ?sp ?override_configfile n
+
+let set_default_max_volatile_sessions_per_subnet ?sp ?override_configfile n =
+  set_default_max_service_sessions_per_subnet ?sp ?override_configfile n;
+  set_default_max_volatile_data_sessions_per_subnet ?sp ?override_configfile n
+
+let set_default_max_anonymous_services_per_session
+    ?sp ?(override_configfile = false) n =
+  let sitedata = 
+    find_sitedata "set_default_max_anonymous_services_per_session" sp 
+  in
+  let b = snd sitedata.Eliom_common.max_anonymous_services_per_session in
+  if override_configfile || not b
+  then 
+    sitedata.Eliom_common.max_anonymous_services_per_session <- (n, b)
+
+let set_default_max_anonymous_services_per_subnet
+    ?sp ?(override_configfile = false) n =
+  let sitedata = 
+    find_sitedata "set_default_max_anonymous_services_per_subnet" sp 
+  in
+  let b = snd sitedata.Eliom_common.max_anonymous_services_per_subnet in
+  if override_configfile || not b
+  then 
+    sitedata.Eliom_common.max_anonymous_services_per_subnet <- (n, b)
+
+
+let set_ipv4_subnet_mask ?sp ?(override_configfile = false) n =
+  let sitedata = 
+    find_sitedata "set_ipv4_subnet_mask" sp 
+  in
+  let b = snd sitedata.Eliom_common.ipv4mask in
+  if override_configfile || not b
+  then 
+    sitedata.Eliom_common.ipv4mask <- (Some n, b)
+
+let set_ipv6_subnet_mask ?sp ?(override_configfile = false) n =
+  let sitedata = 
+    find_sitedata "set_ipv6_subnet_mask" sp 
+  in
+  let b = snd sitedata.Eliom_common.ipv6mask in
+  if override_configfile || not b
+  then 
+    sitedata.Eliom_common.ipv6mask <- (Some n, b)
+
+
+
+
+let set_max_service_sessions_for_group_or_subnet ?session_name ?secure ~sp m =
+  let c =
+    Eliommod_sersess.find_or_create_service_cookie
+      ?session_name ~secure ~sp ()
+  in
+  Eliommod_sessiongroups.Data.set_max c.Eliom_common.sc_session_group_node m
+
+let set_max_volatile_data_sessions_for_group_or_subnet ?session_name ?secure ~sp m =
+  let c =
+    Eliommod_datasess.find_or_create_data_cookie ?session_name ~secure ~sp ()
+  in
+  Eliommod_sessiongroups.Data.set_max c.Eliom_common.dc_session_group_node m
+
+let set_max_volatile_sessions_for_group_or_subnet ?session_name ?secure ~sp m =
+  set_max_service_sessions_for_group_or_subnet ?session_name ?secure ~sp m;
+  set_max_volatile_data_sessions_for_group_or_subnet ?session_name ?secure ~sp m
 
 
 
 
 
 (* expiration dates *)
-
 let set_service_session_cookie_exp_date ?session_name ?secure ~sp t =
   let c = 
     Eliommod_sersess.find_or_create_service_cookie ?session_name ~secure ~sp () 
@@ -483,7 +672,7 @@ let get_service_session_cookie_exp_date ?session_name ?secure ~sp () =
     let (_, _, _, _, exp) = find_service_cookie_only ?session_name ~secure ~sp () in
   let exp = c.Eliom_common.sc_cookie_exp in
     !exp
-  with Not_found -> Eliom_common.CEBrowser
+  with Not_found | Eliom_common.Eliom_Session_expired -> Eliom_common.CEBrowser
 *)
 
 let set_volatile_data_session_cookie_exp_date ?session_name ?secure ~sp t =
@@ -501,7 +690,7 @@ let get_volatile_data_session_cookie_exp_date ?session_name ?secure ~sp () =
     let c = find_data_cookie_only ?session_name ~secure ~sp () in
     let exp = c.Eliom_common.dc_cookie_exp in
     !exp
-  with Not_found -> Eliom_common.CEBrowser
+  with Not_found | Eliom_common.Eliom_Session_expired -> Eliom_common.CEBrowser
 *)
 
 let set_volatile_session_cookies_exp_date ?session_name ?secure ~sp t =
@@ -524,7 +713,10 @@ let get_persistent_data_session_cookie_exp_date ?session_name ?secure ~sp () =
       Eliommod_persess.find_persistent_cookie_only
         ?session_name ~secure ~sp () >>= fun c ->
       return !(c.Eliom_common.pc_cookie_exp))
-    (function Not_found -> return Eliom_common.CEBrowser | e -> fail e)
+    (function
+       | Not_found | Eliom_common.Eliom_Session_expired -> 
+           return Eliom_common.CEBrowser
+       | e -> fail e)
 
 
 
@@ -539,11 +731,13 @@ let get_request ~sp = sp.Eliom_common.sp_request
 let get_ri ~sp = sp.Eliom_common.sp_request.Ocsigen_extensions.request_info
 let get_config_info ~sp = sp.Eliom_common.sp_request.Ocsigen_extensions.request_config
 
-let get_tmp_filename fi = fi.Ocsigen_extensions.tmp_filename
-let get_filesize fi = fi.Ocsigen_extensions.filesize
-let get_original_filename fi = fi.Ocsigen_extensions.original_basename
+let get_tmp_filename fi = fi.Ocsigen_lib.tmp_filename
+let get_filesize fi = fi.Ocsigen_lib.filesize
+let get_original_filename fi = fi.Ocsigen_lib.original_basename
+
+let get_global_table ~sp = 
+  sp.Eliom_common.sp_sitedata.Eliom_common.global_services
 
-let get_global_table ~sp = sp.Eliom_common.sp_sitedata.Eliom_common.global_services
 let get_sitedata ~sp = sp.Eliom_common.sp_sitedata
 
 (** If the session does not exist, we create it
@@ -554,6 +748,15 @@ let get_session_service_table ?session_name ?secure ~sp () =
   in
   c.Eliom_common.sc_table
 
+(** If the session does not exist, we raise Not_found *)
+let get_session_service_table_if_exists ?session_name ?secure ~sp () =
+  try
+    let c = 
+      Eliommod_sersess.find_service_cookie_only ?session_name ~secure ~sp () 
+    in
+    c.Eliom_common.sc_table
+  with Eliom_common.Eliom_Session_expired -> raise Not_found
+
 
 let set_site_handler sitedata handler =
   sitedata.Eliom_common.exn_handler <- handler
@@ -728,7 +931,8 @@ module Session_admin = struct
        float option ref        (* expiration date by timeout
                                   (server side) *) *
        Eliom_common.timeout ref    (* user timeout *) *
-       Eliommod_sessiongroups.sessgrp option ref       (* session group *)) *
+       Eliom_common.sessgrp ref (* session group *) *
+       string Ocsigen_cache.Dlist.node) *
        Eliom_common.sitedata
 
   type data_session =
@@ -738,7 +942,8 @@ module Session_admin = struct
        float option ref        (* expiration date by timeout
                                   (server side) *) *
        Eliom_common.timeout ref   (* user timeout *) *
-       Eliommod_sessiongroups.sessgrp option ref      (* session group *)) *
+       Eliom_common.sessgrp ref (* session group *) *
+       string Ocsigen_cache.Dlist.node) *
        Eliom_common.sitedata
 
   type persistent_session =
@@ -748,22 +953,22 @@ module Session_admin = struct
        float option            (* expiration date by timeout
                                   (server side) *) *
        Eliom_common.timeout        (* user timeout *) *
-       Eliommod_sessiongroups.perssessgrp option           (* session group *))
+       Eliom_common.perssessgrp option           (* session group *))
 
 
   let close_service_session ?(close_group = false)
-      ~session:(cookie, (_, _, _, _, sgr), sitedata) =
+      ~session:(cookie, (_, _, _, _, sgr, sgrnode), sitedata) =
     if close_group then
-      Eliommod_sersess.close_service_group sitedata !sgr
+      Eliommod_sersess.close_service_group !sgr
     else
-      Eliom_common.close_service_session2 sitedata !sgr cookie
+      Eliommod_sessiongroups.Serv.remove sgrnode
 
   let close_volatile_data_session ?(close_group = false)
-      ~session:(cookie, (_, _, _, sgr), sitedata) =
+      ~session:(cookie, (_, _, _, sgr, sgrnode), sitedata) =
     if close_group then
-      Eliommod_datasess.close_data_group sitedata !sgr
+      Eliommod_datasess.close_data_group !sgr
     else
-      Eliommod_datasess.close_data_session2 sitedata !sgr cookie
+      Eliommod_sessiongroups.Data.remove sgrnode
 
   let close_persistent_data_session ?(close_group = false)
       ~session:(cookie, (_, _, _, sg)) =
@@ -784,12 +989,12 @@ module Session_admin = struct
   let remove_persistent_session_data ~session:(cookie, _) ~table =
     Ocsipersist.remove table cookie
 
-  let get_service_session_name ~session:(_, (s, _, _, _, _), _) =
+  let get_service_session_name ~session:(_, (s, _, _, _, _, _), _) =
     try
       Some (snd (Ocsigen_lib.sep '|' s))
     with Not_found -> None
 
-  let get_volatile_data_session_name ~session:(_, (s, _, _, _), _) =
+  let get_volatile_data_session_name ~session:(_, (s, _, _, _, _), _) =
     try
       Some (snd (Ocsigen_lib.sep '|' s))
     with Not_found -> None
@@ -799,12 +1004,12 @@ module Session_admin = struct
       Some (snd (Ocsigen_lib.sep '|' s))
     with Not_found -> None
 
-  let set_service_session_timeout ~session:(_, (_, _, _, r, _), _) t =
+  let set_service_session_timeout ~session:(_, (_, _, _, r, _, _), _) t =
     match t with
     | None -> r := TNone
     | Some t -> r := TSome t
 
-  let set_volatile_data_session_timeout ~session:(_, (_, _, r, _), _) t =
+  let set_volatile_data_session_timeout ~session:(_, (_, _, r, _, _), _) t =
     match t with
     | None -> r := TNone
     | Some t -> r := TSome t
@@ -820,20 +1025,20 @@ module Session_admin = struct
       cookie
       (fullsessname, exp, ti, sessgrp)
 
-  let get_service_session_timeout ~session:(_, (_, _, _, r, _), _) =
+  let get_service_session_timeout ~session:(_, (_, _, _, r, _, _), _) =
     !r
 
-  let get_volatile_data_session_timeout ~session:(_, (_, _, r, _), _) =
+  let get_volatile_data_session_timeout ~session:(_, (_, _, r, _, _), _) =
     !r
 
   let get_persistent_data_session_timeout ~session:(_, (_, _, r, _)) =
     r
 
 
-  let unset_service_session_timeout ~session:(_, (_, _, _, r, _), _) =
+  let unset_service_session_timeout ~session:(_, (_, _, _, r, _, _), _) =
     r := TGlobal
 
-  let unset_volatile_data_session_timeout ~session:(cookie, (_, _, r, _), _) =
+  let unset_volatile_data_session_timeout ~session:(cookie, (_, _, r, _, _), _) =
     r := TGlobal
 
   let unset_persistent_data_session_timeout
@@ -853,9 +1058,7 @@ module Session_admin = struct
 
   (** Iterator on data sessions *)
   let iter_volatile_data_sessions ?sp f =
-    let sitedata =
-      find_sitedata "Admin.iter_volatile_data_sessions" sp
-    in
+    let sitedata = find_sitedata "Admin.iter_volatile_data_sessions" sp in
     Eliommod_sessexpl.iter_data_sessions sitedata f
 
   (** Iterator on persistent sessions *)
