@@ -1,9 +1,5 @@
-val close_data_session2 :
-  Eliom_common.sitedata ->
-  Eliommod_sessiongroups.sessgrp option ->
-  Eliom_common.SessionCookies.key -> unit
 val close_data_group :
-  Eliom_common.sitedata -> Eliommod_sessiongroups.sessgrp option -> unit
+  Eliom_common.sessgrp -> unit
 val close_data_session :
   ?close_group:bool ->
   ?session_name:string -> 
@@ -11,13 +7,13 @@ val close_data_session :
   sp:Eliom_common.server_params -> unit -> unit
 val new_data_cookie :
   Eliom_common.sitedata ->
-  Eliommod_sessiongroups.sessgrp option ->
+  Eliom_common.sessgrp ->
   'a ->
   ('a * float option ref * Eliom_common.timeout ref *
-   Eliommod_sessiongroups.sessgrp option ref)
+   Eliom_common.sessgrp ref * string Ocsigen_cache.Dlist.node)
   Eliom_common.SessionCookies.t -> Eliom_common.one_data_cookie_info
 val find_or_create_data_cookie :
-  ?session_group:string ->
+  ?set_session_group:string ->
   ?session_name:string ->
   secure:bool option ->
   sp:Eliom_common.server_params -> unit -> Eliom_common.one_data_cookie_info
