@@ -31,7 +31,7 @@ val get_eliom_extension : unit -> eliom_extension_sig
 val run_eliom_extension :
   eliom_extension_sig ->
   float ->
-  Ocsigen_extensions.request * Eliom_common.sess_info * 'b *
+  Ocsigen_extensions.request * Eliom_common.sess_info *
   Eliom_common.tables Eliom_common.cookie_info ->
   Eliom_common.sitedata ->
   Ocsigen_extensions.answer Lwt.t
