@@ -1,19 +1,18 @@
 val def_handler : 'a -> exn -> 'b Lwt.t
 val handle_site_exn :
   exn ->
-  Ocsigen_extensions.request * Eliom_common.sess_info * 'a *
+  Ocsigen_extensions.request * Eliom_common.sess_info *
   Eliom_common.tables Eliom_common.cookie_info ->
-  Eliom_common.sitedata -> Eliom_common.result_to_send Lwt.t
+  Eliom_common.sitedata -> Ocsigen_http_frame.result Lwt.t
 val execute :
   float ->
   (float ->
-   Ocsigen_extensions.request * Eliom_common.sess_info * 'a *
+   Ocsigen_extensions.request * Eliom_common.sess_info *
      Eliom_common.tables Eliom_common.cookie_info ->
-   Eliom_common.sitedata -> Eliom_common.result_to_send Lwt.t) ->
-  Ocsigen_extensions.request * Eliom_common.sess_info * 'a *
+   Eliom_common.sitedata -> Ocsigen_http_frame.result Lwt.t) ->
+  Ocsigen_extensions.request * Eliom_common.sess_info *
      Eliom_common.tables Eliom_common.cookie_info ->
-  Eliom_common.sitedata -> Eliom_common.result_to_send Lwt.t
-val compute_exn : string list -> exn list
+  Eliom_common.sitedata -> Ocsigen_http_frame.result Lwt.t
 val gen :
   Eliommod_extensions.eliom_extension_sig option ->
   Eliom_common.sitedata ->
