@@ -22,12 +22,13 @@
 
 
 let (>>=) = Lwt.bind
+let (!!) = Lazy.force
 
 let make_full_group_name ri site_dir_string ipv4mask ipv6mask = function
   | None -> (site_dir_string, 
              Ocsigen_lib.Right 
                (Ocsigen_lib.network_of_ip
-                  (Lazy.force ri.Ocsigen_extensions.ri_remote_ip_parsed)
+                  (!!(ri.Ocsigen_extensions.ri_remote_ip_parsed))
                   ipv4mask
                   ipv6mask
                ))
@@ -92,29 +93,30 @@ struct
           then GroupTable.remove grouptable sess_grp
       | None -> ()
 
+  let get_cl ?set_max sitedata sess_grp =
+    try
+      let cl = find sess_grp in
+      (match set_max with
+        | None -> ()
+        | Some v -> ignore (Ocsigen_cache.Dlist.set_maxsize cl v));
+      cl
+    with Not_found ->
+      let size = match set_max, sess_grp with
+        | None, (_, Ocsigen_lib.Left _) -> A.maxgroup sitedata
+        | None, (_, Ocsigen_lib.Right _) -> A.maxip sitedata
+        | Some v, _ -> v
+      in
+      let cl = Ocsigen_cache.Dlist.create size in
+      Ocsigen_cache.Dlist.set_finaliser
+        (fun node ->
+          A.close_session sitedata (Ocsigen_cache.Dlist.value node);
+          remove_if_empty sess_grp node)
+        cl;
+      GroupTable.add grouptable sess_grp cl;
+      cl
+
   let add ?set_max sitedata sess_id sess_grp =
-    let cl = 
-      try
-        let cl = find sess_grp in
-        (match set_max with
-           | None -> ()
-           | Some v -> ignore (Ocsigen_cache.Dlist.set_maxsize cl v));
-        cl
-      with Not_found ->
-        let size = match set_max, sess_grp with
-          | None, (_, Ocsigen_lib.Left _) -> A.maxgroup sitedata
-          | None, (_, Ocsigen_lib.Right _) -> A.maxip sitedata
-          | Some v, _ -> v
-        in
-        let cl = Ocsigen_cache.Dlist.create size in
-        Ocsigen_cache.Dlist.set_finaliser
-          (fun node ->
-             A.close_session sitedata (Ocsigen_cache.Dlist.value node);
-             remove_if_empty sess_grp node)
-          cl;
-        GroupTable.add grouptable sess_grp cl;
-        cl
-    in
+    let cl = get_cl ?set_max sitedata sess_grp in
     ignore (Ocsigen_cache.Dlist.add sess_id cl);
     match Ocsigen_cache.Dlist.newest cl with
       | Some v -> v
@@ -126,11 +128,14 @@ struct
   let up node =
     Ocsigen_cache.Dlist.up node
 
-  let move ?set_max sitedata node grp2 =
+  let move ?set_max sitedata node sess_grp =
 (*    if set_max <> None || grp1 <> grp2 then begin *)
     let sess_id = Ocsigen_cache.Dlist.value node in
-    remove node;
-    add ?set_max sitedata sess_id grp2
+    let cl = get_cl ?set_max sitedata sess_grp in
+    ignore (Ocsigen_cache.Dlist.move node cl);
+    match Ocsigen_cache.Dlist.newest cl with
+      | Some v -> v
+      | None -> assert false
 (*    end
     else [] *)
 
@@ -195,8 +200,12 @@ let cut n l =
 
 module Pers = struct
 
-  let grouptable : (nbmax * string list) Ocsipersist.table =
-    Ocsipersist.open_table "__eliom_session_group_table"
+  let grouptable : (nbmax * string list) Ocsipersist.table Lazy.t =
+    lazy (Ocsipersist.open_table "__eliom_session_group_table")
+      (* It is lazy because if the module is linked statically,
+         the creation of the table must happen after initialisation
+         of ocsipersist (after reading the configuration file to know
+         the location of the table) *)
 
   let find g =
     match g with
@@ -204,7 +213,7 @@ module Pers = struct
     | Some g ->
         Lwt.catch
           (fun () ->
-             Ocsipersist.find grouptable g >>= fun (_, a) ->
+             Ocsipersist.find !!grouptable g >>= fun (_, a) ->
              Lwt.return a)
           (function
             | Not_found -> Lwt.return []
@@ -215,7 +224,7 @@ module Pers = struct
     | Some sg ->
         Lwt.catch
           (fun () ->
-            Ocsipersist.find grouptable sg >>= fun (max2, cl) ->
+            Ocsipersist.find !!grouptable sg >>= fun (max2, cl) ->
             let max, newmax = match set_max with
               | None -> ((match max2 with
                             | Default -> defaultmax
@@ -225,7 +234,7 @@ module Pers = struct
               | Some (Some v) -> Some v, Val v
             in
             let cl, toclose = cut max cl in
-            Ocsipersist.replace_if_exists grouptable sg (newmax, (sess_id::cl))
+            Ocsipersist.replace_if_exists !!grouptable sg (newmax, (sess_id::cl))
             >>= fun () ->
             Lwt.return toclose)
           (function
@@ -235,7 +244,7 @@ module Pers = struct
                   | Some None -> Nolimit
                   | Some (Some v) -> Val v
                 in
-                Ocsipersist.add grouptable sg (max, [sess_id]) >>= fun () ->
+                Ocsipersist.add !!grouptable sg (max, [sess_id]) >>= fun () ->
                 Lwt.return []
             | e -> Lwt.fail e)
     | None -> Lwt.return []
@@ -245,11 +254,11 @@ module Pers = struct
     | Some sg ->
         Lwt.catch
           (fun () ->
-             Ocsipersist.find grouptable sg >>= fun (max, cl) ->
+             Ocsipersist.find !!grouptable sg >>= fun (max, cl) ->
              let newcl = Ocsigen_lib.list_remove_first_if_any sess_id cl in
              (match newcl with
-                | [] -> Ocsipersist.remove grouptable sg
-                | _ -> Ocsipersist.replace_if_exists grouptable sg (max, newcl)
+                | [] -> Ocsipersist.remove !!grouptable sg
+                | _ -> Ocsipersist.replace_if_exists !!grouptable sg (max, newcl)
              )
           )
           (function
@@ -259,7 +268,7 @@ module Pers = struct
 
   let remove_group sess_grp =
     match sess_grp with
-    | Some sess_grp -> Ocsipersist.remove grouptable sess_grp
+    | Some sess_grp -> Ocsipersist.remove !!grouptable sess_grp
     | None -> Lwt.return ()
 
   let up sess_id grp =
@@ -268,9 +277,9 @@ module Pers = struct
       | Some sg ->
           Lwt.catch
             (fun () ->
-               Ocsipersist.find grouptable sg >>= fun (max, cl) ->
+               Ocsipersist.find !!grouptable sg >>= fun (max, cl) ->
                let newcl = Ocsigen_lib.list_remove_first_if_any sess_id cl in
-               Ocsipersist.replace_if_exists grouptable sg (max, sess_id::newcl)
+               Ocsipersist.replace_if_exists !!grouptable sg (max, sess_id::newcl)
             )
             (function
                | Not_found -> Lwt.return ()
@@ -283,6 +292,6 @@ module Pers = struct
     end
     else Lwt.return []
 
-  let length () = Ocsipersist.length grouptable
+  let length () = Ocsipersist.length !!grouptable
 
 end
