@@ -62,16 +62,18 @@ let fold_read_lines f accum inchnl =
   loop accum
 
 let with_open_out fname f =
-  let oc = Lwt_chan.open_out fname in
+  Lwt_chan.open_out fname >>=
+  (fun oc ->
   finally
     (fun () -> Lwt_chan.flush oc >>= (fun () -> Lwt_chan.close_out oc))
-    f oc
+    f oc)
 
 let with_open_in fname f =
-  let ic = Lwt_chan.open_in fname in
+  Lwt_chan.open_in fname >>=
+  (fun ic ->
   finally
     (fun () -> Lwt_chan.close_in ic)
-    f ic
+    f ic)
 
 let wiki_file_dir =
   let rec find_wikidata = function
