@@ -93,7 +93,7 @@ let parse_config = function
                      let credentials =
                        Http_headers.find
                          (Http_headers.name "Authorization")
-                         ri.request_info.ri_http_frame.Ocsigen_http_frame.header.Ocsigen_http_frame.Http_header.headers
+                         ri.request_info.ri_http_frame.Ocsigen_http_frame.frame_header.Ocsigen_http_frame.Http_header.headers
                      in
                      let encoded =
                        let n = String.length credentials in
