@@ -268,10 +268,12 @@ module File_content =
 
     let result_of_content ?options (c, charset_assoc, mime_assoc) =
       (* open the file *)
-      try
+      catch
+        (fun () ->
         let fdu = Unix.openfile c [Unix.O_RDONLY;Unix.O_NONBLOCK] 0o666 in
         let fd = Lwt_unix.of_unix_file_descr fdu in
-        try
+        catch
+          (fun () ->
           let st = Unix.LargeFile.fstat fdu in
           let etag = get_etag_aux st in
           let stream = read_file fd in
@@ -290,13 +292,12 @@ module File_content =
                    ~finalize:
                    (fun () ->
                       Ocsigen_messages.debug2 "closing file";
-                      Lwt_unix.close fd;
-                      return ())
+                      Lwt_unix.close fd)
                    stream,
                  Some (skip fd))
-            }
-        with e -> Lwt_unix.close fd; raise e
-      with e -> Ocsigen_messages.debug2 (Printexc.to_string e);  fail e
+            })
+          (fun e -> Lwt_unix.close fd >>= (fun () -> fail e)))
+        (fun e -> Ocsigen_messages.debug2 (Printexc.to_string e); fail e)
 
   end
 
