@@ -25,12 +25,15 @@
 open Simplexmlparser
 open Ocsigen_config
 
-let int_of_string tag s =
+let blah_of_string f tag s =
   try
-    int_of_string (Ocsigen_lib.remove_spaces s 0 ((String.length s) -1))
+    f (Ocsigen_lib.remove_spaces s 0 ((String.length s) -1))
   with Failure _ -> raise (Ocsigen_config.Config_file_error
                              ("While parsing <"^tag^"> - "^s^
-                                " is not a valid integer."))
+                                " is not a valid value."))
+
+let int_of_string = blah_of_string int_of_string
+let float_of_string = blah_of_string float_of_string
 
 (*****************************************************************************)
 let default_default_hostname =
@@ -208,39 +211,54 @@ let correct_hostname =
   let regexp = Netstring_pcre.regexp "^[a-zA-Z0-9]+((\\.|-)[a-zA-Z0-9]+)*$" in
   fun h -> Netstring_pcre.string_match regexp h 0 <> None
 
-(* Split the [host] field, first according to spaces, and then according
-   to wildcards '*' *)
-let parse_host_field host =
-  match host with
-    | None -> [[Ocsigen_extensions.Wildcard], None] (* default = "*:*" *)
-    | Some s ->
-        let parse_one_host ss =
-          let host, port =
-            try
-              let dppos = String.index ss ':'
-              and len = String.length ss in
-              let host = String.sub ss 0 dppos
-              and port =
-                match String.sub ss (dppos+1) ((len - dppos) - 1) with
-                  | "*" -> None
-                  | p -> Some (int_of_string "host" p)
-              in host, port
-            with
-              | Not_found -> ss, None
-              | Failure _ ->
-                  raise (Config_file_error "bad port number")
-          in
-          let split_host = function
-            | Netstring_str.Delim _ -> Ocsigen_extensions.Wildcard
-            | Netstring_str.Text t ->
-                Ocsigen_extensions.Text (t, String.length t)
-          in
-          List.map split_host
-            (Netstring_str.full_split (Netstring_str.regexp "[*]+") host),
-          port
-        in
-        List.map parse_one_host
-          (Netstring_str.split (Netstring_str.regexp "[ \t]+") s)
+(* Splits the [host] field, first according to spaces
+   (which encode disjunction),
+   and then according to wildcards '*' ; we then transform the hosts-with-regexp
+   into a regepx that matches a potential host. 
+   The whole result is cached because user config files (for userconf)
+   are read at each request. *)
+let parse_host_field =
+  let h = Hashtbl.create 17 in
+  (fun (hostfilter : string option) ->
+     try Hashtbl.find h hostfilter
+     with Not_found ->
+       let r = match hostfilter with
+         | None -> ["*", Netstring_pcre.regexp ".*$", None] (* default = "*:*" *)
+         | Some s ->
+             let parse_one_host ss =
+               let host, port =
+                 try
+                   let dppos = String.index ss ':'
+                   and len = String.length ss in
+                   let host = String.sub ss 0 dppos
+                   and port =
+                     match String.sub ss (dppos+1) ((len - dppos) - 1) with
+                       | "*" -> None
+                       | p -> Some (int_of_string "host" p)
+                   in host, port
+                 with
+                   | Not_found -> ss, None
+                   | Failure _ ->
+                       raise (Config_file_error "bad port number")
+               in
+               let split_host = function
+                 | Netstring_str.Delim _ -> ".*"
+                 | Netstring_str.Text t -> Netstring_pcre.quote t
+               in
+               (host,
+                Netstring_pcre.regexp
+                  (String.concat ""
+                     ((List.map split_host
+                         (Netstring_str.full_split
+                            (Netstring_str.regexp "[*]+") host))@["$"])),
+                port)
+             in
+             List.map parse_one_host
+               (Netstring_str.split (Netstring_str.regexp "[ \t]+") s)
+       in
+       Hashtbl.add h hostfilter r;
+       (r : Ocsigen_extensions.virtual_hosts)
+  )
 
 
 (* Extract a default hostname from the "host" field if no default is provided *)
@@ -252,7 +270,8 @@ let get_defaulthostname ~defaulthostname ~defaulthttpport ~host =
           (* Something more clever could be envisioned *)
           let rec aux = function
             | [] -> default_default_hostname
-            | ([Ocsigen_extensions.Text (t, _)], (Some 80 | None)) :: _ -> t
+            | (t, _, (Some 80 | None)) :: _ when String.contains t '*' = false ->
+                t
             | _ :: q -> aux q
           in
           let host = aux host in
@@ -342,6 +361,15 @@ let parse_server isreloading c =
       | (Element ("commandpipe" as st, [], p))::ll ->
           set_command_pipe (parse_string_tag st p);
           parse_server_aux ll
+      | (Element ("shutdowntimeout" as st, [], p))::ll ->
+          let p = parse_string_tag st p in
+          let t =
+            if p = "notimeout"
+            then None
+            else Some (float_of_string st p)
+          in
+          set_shutdown_timeout t;
+          parse_server_aux ll
       | (Element ("debugmode", [], []))::ll ->
           set_debugmode true;
           parse_server_aux ll
@@ -372,9 +400,11 @@ let parse_server isreloading c =
           in begin
             match modules with
               | `Files modules ->
-                  Ocsigen_loader.loadfiles (preloadfile l) postloadfile false modules;
+                  Ocsigen_loader.loadfiles
+                    (preloadfile l) postloadfile false modules;
               | `Name name ->
-                  Ocsigen_loader.init_module (preloadfile l) postloadfile false name
+                  Ocsigen_loader.init_module
+                    (preloadfile l) postloadfile false name
           end;
           parse_server_aux ll
       | (Element ("library", atts, l))::ll ->
@@ -461,17 +491,13 @@ let parse_server isreloading c =
               | None, None -> "utf-8"
           in
           let defaulthttpport = match defaulthttpport with
-            | None ->
-                (try snd (List.hd (Ocsigen_config.get_ports ())) 
-                with Failure _ -> 80)
+            | None -> Ocsigen_config.get_default_port ()
             | Some p -> int_of_string "host" p
           in
           let defaulthostname = get_defaulthostname
             ~defaulthostname ~defaulthttpport ~host in
           let defaulthttpsport = match defaulthttpsport with
-            | None ->
-                (try snd (List.hd (Ocsigen_config.get_sslports ())) 
-                with Failure _ -> 443)
+            | None -> Ocsigen_config.get_default_sslport ()
             | Some p -> int_of_string "host" p
           in
           let parse_host = Ocsigen_extensions.parse_config_item host in
@@ -492,6 +518,8 @@ let parse_server isreloading c =
             follow_symlinks = Ocsigen_extensions.FollowSymlinksIfOwnerMatch;
             do_not_serve_404 = serve_everything;
             do_not_serve_403 = serve_everything;
+            uploaddir = Ocsigen_config.get_uploaddir ();
+            maxuploadfilesize = Ocsigen_config.get_maxuploadfilesize ();
           }
           in
           let parse_config =
@@ -543,6 +571,11 @@ let parse_server isreloading c =
   in Ocsigen_extensions.set_hosts (parse_server_aux c)
 
 
+(* Types of socket declarable in configuration file *)
+type socket_type =
+  | IPv4 of Unix.inet_addr
+  | IPv6 of Unix.inet_addr
+  | All
 
 (* Parsing <port> tags *)
 let parse_port =
@@ -554,16 +587,16 @@ let parse_port =
     let do_match r = Netstring_pcre.string_match r s 0 in
     let get x i = Netstring_pcre.matched_group x i s in
     match do_match all_ipv6 with
-      | Some r -> Some (Unix.inet6_addr_any), int_of_string "port" (get r 1)
+      | Some r -> IPv6 (Unix.inet6_addr_any), int_of_string "port" (get r 1)
       | None -> match do_match all_ipv4 with
-      | Some r -> Some (Unix.inet_addr_any), int_of_string "port" (get r 1)
+      | Some r -> IPv4 (Unix.inet_addr_any), int_of_string "port" (get r 1)
       | None -> match do_match single_ipv6 with
-      | Some r -> Some (Unix.inet_addr_of_string (get r 1)), 
+      | Some r -> IPv6 (Unix.inet_addr_of_string (get r 1)),
           int_of_string "port" (get r 2)
       | None -> match do_match single_ipv4 with
-      | Some r -> Some (Unix.inet_addr_of_string (get r 1)), 
+      | Some r -> IPv4 (Unix.inet_addr_of_string (get r 1)),
           int_of_string "port" (get r 2)
-      | None -> None, int_of_string "port" s
+      | None -> All, int_of_string "port" s
 
 
 (* First parsing of config file *)
@@ -664,7 +697,12 @@ let extract_info c =
   in
   ((user, group), si, (mint, maxt))
 
-let parse_config () =
-    parser_config (Ocsigen_config.config ())
+let parse_config ?file () =
+  let file = 
+    match file with
+      | None -> Ocsigen_config.get_config_file ()
+      | Some f -> f
+  in
+  parser_config (Simplexmlparser.xmlparser_file file)
 
 (******************************************************************)
