@@ -251,7 +251,7 @@ let get_request_infos
   Lwt.catch
     (fun () ->
 
-       let (_, headerhost, headerport, url, parsed_url, path, params, get_params) =
+       let (_, headerhost, headerport, url, path, params, get_params) =
          Ocsigen_lib.parse_url url
        in
 
@@ -338,7 +338,6 @@ let get_request_infos
 
        Lwt.return
          {ri_url_string = url;
-          ri_url = parsed_url;
           ri_method = meth;
           ri_protocol = http_frame.Ocsigen_http_frame.frame_header.Ocsigen_http_frame.Http_header.proto;
           ri_ssl = Lwt_ssl.is_ssl (Ocsigen_http_com.connection_fd receiver);
@@ -506,6 +505,8 @@ let handle_result_frame ri res send =
         Ocsigen_range.compute_range ri res
         >>= send
 
+let http_url_syntax = Hashtbl.find Neturl.common_url_syntax "http"
+
 
 let service receiver sender_slot request meth url port sockaddr =
   (* sender_slot is here for pipelining:
@@ -589,7 +590,7 @@ let service receiver sender_slot request meth url port sockaddr =
                      | Ocsigen_http_com.Timeout ->
                          warn sockaddr "timeout while reading contents"
                      | Ocsigen_http_com.Aborted ->
-                         warn sockaddr "reading thread aborted"
+                         dbg sockaddr "reading thread aborted"
                      | Http_error.Http_exception (code, mesg, _) ->
                          warn sockaddr (Http_error.string_of_http_exception e)
                      | _ ->
@@ -668,7 +669,7 @@ let service receiver sender_slot request meth url port sockaddr =
                        urls become correct *)
                     Ocsigen_messages.debug2 "-> Sending 301 Moved permanently";
                     let port = Ocsigen_extensions.get_port request in
-                    let new_url = Neturl.default_url
+                    let new_url = Neturl.make_url
                       ~scheme:(if ri.ri_ssl then "https" else "http")
                       ~host:(Ocsigen_extensions.get_hostname request)
                       ?port:(if (port = 80 && not ri.ri_ssl)
@@ -677,7 +678,8 @@ let service receiver sender_slot request meth url port sockaddr =
                              else Some port)
                       ~path:(""::(Ocsigen_lib.add_end_slash_if_missing
                                     ri.ri_full_path))
-                      (Neturl.remove_from_url ~path:true ri.ri_url)
+                      ?query:ri.ri_get_params_string
+                      http_url_syntax
                     in
                     send_aux {
                       (Ocsigen_http_frame.empty_result ()) with
@@ -782,7 +784,7 @@ let handle_connection port in_ch sockaddr =
     | Ocsigen_http_com.Timeout ->
         warn sockaddr "timeout"
     | Ocsigen_http_com.Aborted ->
-        warn sockaddr "writing thread aborted"
+        dbg sockaddr "writing thread aborted"
     | Ocsigen_stream.Interrupted e' ->
         warn sockaddr ("interrupted content stream (" ^ string_of_exn e' ^ ")")
     | _ ->
@@ -812,7 +814,7 @@ let handle_connection port in_ch sockaddr =
         Ocsigen_http_com.abort receiver;
         Ocsigen_http_com.wait_all_senders receiver
     | Ocsigen_http_com.Aborted ->
-        warn sockaddr "reading thread aborted";
+        dbg sockaddr "reading thread aborted";
         Ocsigen_http_com.wait_all_senders receiver
     | Http_error.Http_exception (code, mes, _) ->
         warn sockaddr (Http_error.string_of_http_exception e);
@@ -954,14 +956,15 @@ let rec wait_connection use_ssl port socket =
               return ())
          >>= fun () ->
          Ocsigen_messages.debug2 "** CLOSE";
-         begin
-           try
-             Lwt_unix.close s
-           with Unix.Unix_error _ as e ->
-             Ocsigen_messages.unexpected_exception
-               e "Server.wait_connection (close)"
-         end;
-         decr_connected ()
+         Lwt.catch
+           (fun () -> Lwt_unix.close s)
+	   (function
+	   | Unix.Unix_error _ as e ->
+	     Ocsigen_messages.unexpected_exception e
+               "Server.wait_connection (close)";
+             Lwt.return ()
+           | e -> Lwt.fail e)
+	 >>= (fun () -> decr_connected ())
        in
 
        Lwt_util.iter handle_one l >>= fun () ->
