@@ -39,10 +39,12 @@ type link = [ = `Link ];
 type script = [ = `Script ];
 type input = [ = `Input ];
 type textarea = [ = `Textarea ];
+type param = [ = `Param ];
 type select = [ = `Select ];
 type selectoption = [ = `Option ];
 type optgroup = [ = `Optgroup ];
 type button = [ = `Button ];
+type ruby = [ = `Ruby_simple1 | `Ruby_simple2 | `Ruby_complex ];
 type pcdata = [ = `PCDATA ];
 type notag;
 type html_content = [ = `Body | `Head | `Frameset ];
@@ -196,7 +198,7 @@ type select_attrib =
   ];
 type optgroup_attrib = [ = common | `Disabled | `Label ];
 type option_attrib = [ = common | `Selected | `Value | `Disabled | `Label ];
-type button_attrib = [ = common | `Name | `Value | `Button_Type ];
+type button_attrib = [ = common | `Name | `Value | `Button_Type | `Disabled | `Accesskey | `Tabindex ];
 type fieldset_attrib = common;
 type label_attrib = [ = common | `Accesskey | `For ];
 type legend_attrib = [ = common | `Accesskey ];
