@@ -109,15 +109,6 @@ module type T =
 (** A space-separated list of character encodings, as per RFC2045 (MIME).
     @see <http://www.ietf.org/rfc/rfc2045.txt> RFC2045 *)
 
-    type color =
-        [ `Aqua | `Black | `Blue | `Fuchsia | `Gray | `Green | `Lime | `Maroon
-        | `Navy | `Olive | `Purple | `Red | `Silver | `Teal | `White | `Yellow
-        | `Hex of string | `RGB of int * int * int ]
-(** The attribute value type [color] refers to color definitions as specified in
-    SRGB.  A color value may either be a hexadecimal number (prefixed by a hash mark)
-    or one of the following sixteen color names. The color names are case-insensitive.
-    @see <http://www.w3.org/Graphics/Color/sRGB> A Standard Default Color Space for the Internet. *)
-
     type contenttype = string
 (** A media type, as per RFC2045 (MIME).
     @see <http://www.ietf.org/rfc/rfc2045.txt> RFC2045 *)
@@ -364,11 +355,41 @@ module type T =
     type i18n = [ `XML_lang ]
     val a_xml_lang : nmtoken -> [>`XML_lang] attrib
 
+
 (** {2 Style}
     The Style collection is deprecated, because the Style Attribute Module is
     deprecated. *)
 
-    type common = [ core | i18n ]
+(** {2 Events} *)
+
+(** Javascript events *)
+
+    type events = [ `OnClick | `OnDblClick | `OnMouseDown | `OnMouseUp |
+                    `OnMouseOver | `OnMouseMove | `OnMouseOut | `OnKeyPress |
+                    `OnKeyDown | `OnKeyUp ]
+
+    val a_onblur : cdata -> [>`OnBlur] attrib
+    val a_onclick : cdata -> [>`OnClick] attrib
+    val a_ondblclick : cdata -> [>`OnDblClick] attrib
+    val a_onchange : cdata -> [>`OnChange] attrib
+    val a_onfocus : cdata -> [>`OnFocus] attrib
+    val a_onload : cdata -> [>`OnLoad] attrib
+    val a_onunload : cdata -> [>`OnUnload] attrib
+    val a_onreset : cdata -> [>`OnReset] attrib
+    val a_onselect : cdata -> [>`OnSelect] attrib
+    val a_onsubmit : cdata -> [>`OnSubmit] attrib
+    val a_onmousedown : cdata -> [>`OnMouseDown] attrib
+    val a_onmouseup : cdata -> [>`OnMouseUp] attrib
+    val a_onmouseover : cdata -> [>`OnMouseOver] attrib
+    val a_onmousemove : cdata -> [>`OnMouseMove] attrib
+    val a_onmouseout : cdata -> [>`OnMouseOut] attrib
+    val a_onkeypress : cdata -> [>`OnKeyPress] attrib
+    val a_onkeydown : cdata -> [>`OnKeyDown] attrib
+    val a_onkeyup : cdata -> [>`OnKeyUp] attrib
+
+
+    type common = [ core | i18n | events | `Style_Attr]
+
 
 (** {1 Modules, Element Sets and Attributes } *)
 
@@ -556,6 +577,8 @@ module type T =
     attribute is not set, the initial value is set to the contents of
     the [option] element. *)
 
+    val a_value_type : [< `Data | `Ref | `Object ] -> [>`Value_Type] attrib
+
 
 (** {3 5.5.2. Forms Module} *)
 
@@ -602,13 +625,7 @@ module type T =
     val a_rules : [< `None | `Groups | `Rows | `Cols | `All ] -> [>`Rules] attrib
     val a_char : character -> [>`Char] attrib
     val a_charoff : length -> [>`Charoff] attrib
-
-(** {2 5.7. Image Module} *)
-
-    module IMAGE :
-        sig
-          type inline = [ `Img ]
-        end
+    val a_span : number -> [>`Span] attrib
 
     val a_alt : text -> [>`Alt] attrib
     val a_height : length -> [>`Height] attrib
@@ -624,6 +641,7 @@ module type T =
     val a_usemap : idref -> [>`Usemap] attrib
 
 (** {2 5.9. Server-side Image Map Module} *)
+    val a_ismap : [< `Ismap ] -> [>`Ismap] attrib
 
 (** {2 5.10. Object Module} *)
 
@@ -651,7 +669,6 @@ module type T =
 
     val a_target : frametarget -> [>`Target] attrib
 
-(** {2 5.13. Iframe Module} *)
 
 (** {2 5.14. Intrinsic Events Module} *)
 
@@ -683,6 +700,8 @@ module type T =
 
 (** {2 5.18. Style Attribute Module} *)
 
+    val a_style : string -> [>`Style_Attr] attrib
+
 (** {2 5.19. Link Module} *)
 
     module LINK :
@@ -719,7 +738,15 @@ module type T =
 
 (* VB *)
 
+(* CH *)
+    module RUBY : sig
+      type inline = [ `Ruby_simple1 | `Ruby_simple2 | `Ruby_complex ]
+      type flow = inline
+    end
 
+    type no_ruby_inline = [ TEXT.inline | PRESENTATION.inline | HYPERTEXT.inline | SPECIAL.inline | FORMS.inline | i18nclass ]
+    type no_ruby_content = [ `PCDATA | no_ruby_inline | misc ]
+(* CH *)
 
 (** {1 Combined Element Sets:} *)
 
@@ -729,30 +756,31 @@ module type T =
         [ TEXT.block | PRESENTATION.block | TABLES.block | TEXT.heading | LIST.list | misc ]
 
     type flow =
-        [ TEXT.flow | HYPERTEXT.flow | LIST.flow | FORMS.flow | TABLES.flow | PRESENTATION.flow | SPECIAL.flow | i18nclass | misc ]
+        [ TEXT.flow | HYPERTEXT.flow | LIST.flow | FORMS.flow | TABLES.flow | PRESENTATION.flow | SPECIAL.flow | i18nclass | misc | RUBY.flow ]
     type flow_sans_table =
-        [ TEXT.flow | HYPERTEXT.flow | LIST.flow | FORMS.flow | PRESENTATION.flow | SPECIAL.flow | i18nclass | misc ]
+        [ TEXT.flow | HYPERTEXT.flow | LIST.flow | FORMS.flow | PRESENTATION.flow | SPECIAL.flow | i18nclass | misc | RUBY.flow ]
 
     type inline =
         [ TEXT.inline | HYPERTEXT.inline | PRESENTATION.inline
-        | FORMS.inline | IMAGE.inline | SPECIAL.inline | i18nclass | misc ]
+        | FORMS.inline | SPECIAL.inline | i18nclass | misc | RUBY.inline ]
 
     type inline_sans_a_mix =
         [ TEXT.inline | PRESENTATION.inline
-        | FORMS.inline | IMAGE.inline | SPECIAL.inline | i18nclass |misc ]
+        | FORMS.inline | SPECIAL.inline | i18nclass | misc | RUBY.inline ]
 
     type buttoncontent = (* VB *)
         [ TEXT.inline | PRESENTATION.inline
-      | IMAGE.inline | SPECIAL.inline | i18nclass | block_sans_form ]
+        | SPECIAL.inline | i18nclass | block_sans_form ]
     type precontent = (* VB *)
         [ TEXT.inline | HYPERTEXT.inline | `Tt | `I | `B | `Script | `Map |
           i18nclass ]
     type inline_sans_label =
         [ TEXT.inline | HYPERTEXT.inline | PRESENTATION.inline
-        | FORMS.inline_sans_label | IMAGE.inline | SPECIAL.inline | i18nclass | misc ]
+        | FORMS.inline_sans_label | SPECIAL.inline | i18nclass | misc ]
 
     type heading = TEXT.heading
 
+
 (** {1 Elements} *)
 
     (* For Ocsigen I need to specify the variance --Vincent *)
@@ -764,6 +792,10 @@ module type T =
     type ('a, 'b, 'c) unary = ?a:('a attrib list) -> 'b elt -> 'c elt
     type ('a, 'b, 'c, 'd) binary = ?a:('a attrib list) -> 'b elt -> 'c elt -> 'd elt
 
+(* CH *)
+    type ('a, 'b, 'c, 'd, 'e, 'f) quadry= ?a:('a attrib list) -> 'b elt -> 'c elt -> 'd elt -> 'e elt -> 'f elt
+(* CH *)
+
     type ('a, 'b, 'c) star = ?a:('a attrib list) -> 'b elt list -> 'c elt
 (** Star '*' denotes any number of children, uncluding zero. *)
 
@@ -781,7 +813,7 @@ module type T =
         [< `Meta | `Link | `Style | `Object | `Script ] elt list
         -> [>`Head] elt
     val title : ([< i18n ], [< `PCDATA ], [>`Title]) unary
-    val body : ([< common ], [< block ], [>`Body]) star
+    val body : ([< common |`OnLoad |`OnUnload ], [< block ], [>`Body]) star
 
 (** {2 Data} *)
 
@@ -832,7 +864,7 @@ module type T =
 
     val a : ([< common | `Accesskey | `Charset | `Href | `Hreflang
              | `Name_01_00 | `Rel | `Rev | `Tabindex | `Target | `Type
-             | `Shape | `Coords ],
+             | `Shape | `Coords | `OnBlur |`OnFocus],
              [< `PCDATA | inline_sans_a_mix ], [>`A]) star
 
 (** {2 List} *)
@@ -855,15 +887,17 @@ module type T =
     val sup : ([< common ], [< `PCDATA | inline ], [>`Sup]) star
     val tt : ([< common ], [< `PCDATA | inline ], [>`Tt]) star
 
-(* VB *)
-    val bdo : ([< core | `Dir | `XML_lang ],[< `PCDATA | inline ],[> `Bdo ]) star
-    val area : alt:text -> ([< common | `Href | `Shape | `Coords | `Nohref | `Tabindex | `Accesskey ],[>`Area]) nullary
+(* CH *)
+    val bdo : dir:[< `Ltr | `Rtl ] -> ([< core | `XML_lang ],[< `PCDATA | inline ],[> `Bdo ]) star
+(* CH *)
 
-    val map : id:id -> ([< `XMLns | `Class | `Title | i18n ],[< block | `Area ],[>`Map]) plus
+    val area : alt:text -> ([< common | `Href | `Shape | `Coords | `Nohref | `Tabindex | `Accesskey |`OnBlur |`OnFocus],[>`Area]) nullary
+
+    val map : id:id -> ([< events | core | `XMLns | `Class | `Title | i18n ],[< block | `Area ],[>`Map]) plus
 
     val del : ([< common | `Cite | `Datetime ],[< `PCDATA | flow ],[>`Del]) star
     val ins : ([< common | `Cite | `Datetime ],[< `PCDATA | flow ],[>`Ins]) star
-    val script : contenttype:contenttype -> ([< `XMLns | `Charset | `Src | `Defer | `XML_space ],[< `PCDATA ],[>`Script]) unary
+    val script : contenttype:contenttype -> ([< `XMLns | `Id | `Charset | `Src | `Defer | `XML_space ],[< `PCDATA ],[>`Script]) unary
     val noscript : ([< common ],[< block ],[>`Noscript]) plus
 (* VB *)
 
@@ -877,18 +911,18 @@ module type T =
     module Basic_Forms :
         sig
           val form : action:uri ->
-            ([< common | `Enctype | `Method | `Name_01_00 | `Target ],
+            ([< common | `Enctype | `Method | `Name_01_00 | `Target |`OnReset | `OnSubmit],
              [< block_sans_form ], [>`Form]) plus
           val input : ([< common | `Accesskey | `Checked | `Maxlength | `Name | `Size
-                       | `Src | `Tabindex | `Input_Type | `Value | `Usemap ], [>`Input]) nullary
+                       | `Src | `Tabindex | `Input_Type | `Value | `Usemap|`OnBlur |`OnChange |`OnFocus | `OnSelect], [>`Input]) nullary
           val label : ([< common | `Accesskey | `For ],
                        [< `PCDATA | inline_sans_label ], [>`Label]) star
           val option : ([< common | `Selected | `Value ],
                         [< `PCDATA ], [>`Option]) unary
-          val select : ([< common | `Multiple | `Name | `Size | `Tabindex ],
+          val select : ([< common | `Multiple | `Name | `Size | `Tabindex |`OnBlur |`OnChange |`OnFocus ],
                         [< `Option ], [>`Select]) plus
           val textarea : rows:number -> cols:number ->
-            ([< common | `Accesskey | `Name | `Tabindex ],
+            ([< common | `Accesskey | `Name | `Tabindex |`OnBlur |`OnChange |`OnFocus | `OnSelect],
              [< `PCDATA ], [>`Textarea]) unary
         end
 
@@ -896,27 +930,27 @@ module type T =
 
 (** Generic forms. WARNING: If you find a bug or if something is missing please send a bug report to the Ocsigen project! -- VB *)
     val form : action:uri ->
-      ([< common | `Enctype | `Method | `Name_01_00 | `Target | `Accept_charset | `Accept ],
+      ([< common | `Enctype | `Method | `Name_01_00 | `Target | `Accept_charset | `Accept |`OnReset | `OnSubmit],
        [< block_sans_form | `Fieldset ], [>`Form]) plus
     val input : ([< common | `Accesskey | `Checked | `Maxlength | `Name | `Size
-  | `Src | `Tabindex | `Input_Type | `Value | `Disabled | `Readonly | `Alt | `Accept | `Usemap ], [>`Input]) nullary
-    val label : ([< common | `Accesskey | `For ],
+  | `Src | `Tabindex | `Input_Type | `Value | `Disabled | `Readonly | `Alt | `Accept | `Usemap |`Ismap |`OnBlur |`OnChange |`OnFocus | `OnSelect], [>`Input]) nullary
+    val label : ([< common | `Accesskey | `For |`OnBlur |`OnFocus],
                  [< `PCDATA | inline_sans_label ], [>`Label]) star
     val optgroup : label:text ->
       ([< common | `Disabled ],
        [< `Option ], [>`Optgroup]) plus
     val option : ([< common | `Selected | `Value | `Disabled | `Label ],
                   [< `PCDATA ], [>`Option]) unary
-    val select : ([< common | `Multiple | `Name | `Size | `Tabindex | `Disabled ],
+    val select : ([< common | `Multiple | `Name | `Size | `Tabindex | `Disabled |`OnBlur |`OnChange |`OnFocus ],
                   [< `Option | `Optgroup ], [>`Select]) plus
     val textarea : rows:number -> cols:number ->
-      ([< common | `Accesskey | `Name | `Tabindex | `Disabled | `Readonly ],
+      ([< common | `Accesskey | `Name | `Tabindex | `Disabled | `Readonly |`OnBlur |`OnChange |`OnFocus | `OnSelect],
        [< `PCDATA ], [>`Textarea]) unary
     val fieldset : ([< common ],
                     [< `PCDATA | `Legend | flow ], [>`Fieldset]) star
     val legend : ([< common | `Accesskey ],
                     [< `PCDATA | inline ], [>`Legend]) star
-    val button : ([< common | `Name | `Value | `Button_Type ],
+    val button : ([< common | `Name | `Value | `Button_Type | `Disabled | `Accesskey | `Tabindex |`OnBlur |`OnFocus],
                     [< `PCDATA | buttoncontent ], [>`Button]) star
 
 (** {2 Tables} *)
@@ -981,16 +1015,19 @@ module type T =
 (** {2 Image} *)
 
     val img : src:uri -> alt:text ->
-      ([< common | `Height | `Longdesc | `Name_01_00 | `Width | `Usemap ], [>`Img]) nullary
+      ([< common | `Height | `Longdesc | `Name_01_00 | `Width | `Usemap |`Ismap ], [>`Img]) nullary
 
 (** {2 Object} VB *)
 
-    val object_ : ([< common | `Declare | `Classid | `Codebase | `Data | `Type | `Codetype | `Archive | `Standby | `Height | `Width | `Name | `Tabindex ],[< `PCDATA | flow | `Param | `Usemap ],[> `Object ]) star
+    val object_ : ([< common | `Declare | `Classid | `Codebase | `Data | `Type | `Codetype | `Archive | `Standby
+                   | `Height | `Width | `Name | `Tabindex | `Usemap],[< `PCDATA | flow | `Param ],[> `Object ]) star
+
+    val param : name:text ->([< `XMLns |`Id | `Value | `Value_Type | `Type ], [> `Param ]) nullary
 
 (** {2 Frames} *)
 
     val frameset : ?noframes:([< `Noframes ] elt) ->
-      ([< core | `FS_Rows | `FS_Cols ], [< `Frameset | `Frame ], [>`Frameset]) plus
+      ([< core | `FS_Rows | `FS_Cols |`OnLoad |`OnUnload], [< `Frameset | `Frame ], [>`Frameset]) plus
 
     val frame : src:uri ->
       ([< core | `Frameborder | `Longdesc | `Marginheight | `Marginwidth
@@ -1006,7 +1043,7 @@ module type T =
 (** {2 Style Sheets} *)
 
     val style : contenttype:contenttype ->
-      ([< i18n | `Media | `Title | `XML_space ], [< `PCDATA ], [>`Style]) star
+      ([< i18n |`XMLns |`Id | `Media | `Title | `XML_space ], [< `PCDATA ], [>`Style]) star
 
 (** {2 Link} *)
 
@@ -1016,6 +1053,27 @@ module type T =
 (** {2 Base} *)
 
     val base : href:uri -> unit -> [>`Base] elt
+    (*val base : href:uri -> ([`XMLns], [>`Base] elt)*)
+
+(** {2 Ruby} *)
+
+    val ruby_simple1 : ?a:([< common] attrib list) ->
+      [< `Rb ] elt -> [< `Rt ] elt -> [>`Ruby_simple1] elt
+    val ruby_simple2 : ?a:([< common] attrib list) ->
+      [< `Rb ] elt -> [< `Rp ] elt -> [< `Rt ] elt -> [< `Rp ] elt -> [>`Ruby_simple2] elt
+    val ruby_complex : ?a:([< common] attrib list) ->
+      [< `Rbc ] elt -> [< `Rtc_complex ] elt -> [>`Ruby_complex] elt
+
+    val rbc : ([< common ], [< `Rb ], [>`Rbc]) plus
+    val rtc : ([< common ], [< `Rt ], [>`Rtc]) plus
+    val rtc_complex : ([< common ], [< `Rt_complex ], [>`Rtc]) plus
+    val rb : ([< common ], [< no_ruby_content ], [>`Rb]) star
+    val rt : ([< common ], [< no_ruby_content ], [>`Rt]) star
+    val rt_complex : ([< common | `Rbspan], [< no_ruby_content ], [>`Rt]) star
+    val rp : ([< common ], [< `PCDATA ], [>`Rp]) star
+
+    val a_rbspan : number -> [>`Rbspan] attrib
+
 
 (** {1 Output} *)
 
@@ -1126,7 +1184,11 @@ module Version =
     type core = [ `Class | `Id | `Title ]
     type i18n = [ `XML_lang ]
 
-    type common = [ core | i18n ]
+    type events = [ `OnClick | `OnDblClick | `OnMouseDown | `OnMouseUp |
+                    `OnMouseOver | `OnMouseMove | `OnMouseOut | `OnKeyPress |
+                    `OnKeyDown | `OnKeyUp ]
+
+    type common = [ core | i18n | events | `Style_Attr]
 
     type 'a attrib = XML.attrib
     type 'a attribs = XML.attribs
@@ -1150,10 +1212,7 @@ module Version =
     type character = char
     type charset = string
     type charsets = charset list (* space-separated *)
-    type color =
-        [ `Aqua | `Black | `Blue | `Fuchsia | `Gray | `Green | `Lime | `Maroon
-        | `Navy | `Olive | `Purple | `Red | `Silver | `Teal | `White | `Yellow
-        | `Hex of string | `RGB of int * int * int ]
+
     type contenttype = string
     type contenttypes = contenttype list (* comma-separated *)
     type coords = string list (* Comma separated list of coordinates to use in defining areas. *)
@@ -1181,28 +1240,6 @@ module Version =
     let uri_of_string s = s
     let string_of_uri s = s
 
-    let color_attrib name value =
-      string_attrib name
-        (match value with
-        | `Aqua -> "aqua"
-        | `Black -> "black"
-        | `Blue -> "blue"
-        | `Fuchsia -> "fuchsia"
-        | `Gray -> "gray"
-        | `Green -> "green"
-        | `Lime -> "lime"
-        | `Maroon -> "maroon"
-        | `Navy -> "navy"
-        | `Olive -> "olive"
-        | `Purple -> "purple"
-        | `Red -> "red"
-        | `Silver -> "silver"
-        | `Teal -> "teal"
-        | `White -> "white"
-        | `Yellow -> "yellow"
-        | `Hex h -> ("#" ^ h)
-        | `RGB (r, g, b) -> Printf.sprintf "#%02X%02X%02X" r g b)
-
     let length_attrib name = function
       | `Pixels p -> int_attrib name p
       | `Percent p -> string_attrib name (string_of_int p ^ "%")
@@ -1273,6 +1310,29 @@ module Version =
 
     let a_style = string_attrib "style"
 
+
+    (* Events: *)
+
+    let a_onblur = string_attrib "onblur"
+    let a_onclick = string_attrib "onclick"
+    let a_ondblclick = string_attrib "ondblclick"
+    let a_onchange = string_attrib "onchange"
+    let a_onfocus = string_attrib "onfocus"
+    let a_onload = string_attrib "onload"
+    let a_onunload = string_attrib "onunload"
+    let a_onreset = string_attrib "onreset"
+    let a_onselect = string_attrib "onselect"
+    let a_onsubmit = string_attrib "onsubmit"
+    let a_onmousedown = string_attrib "onmousedown"
+    let a_onmouseup = string_attrib "onmouseup"
+    let a_onmouseover = string_attrib "onmouseover"
+    let a_onmousemove = string_attrib "onmousemove"
+    let a_onmouseout = string_attrib "onmouseout"
+    let a_onkeypress = string_attrib "onkeypress"
+    let a_onkeydown = string_attrib "onkeydown"
+    let a_onkeyup = string_attrib "onkeyup"
+
+
     (* Other Attributes *)
 
     let a_profile = string_attrib "profile"
@@ -1310,13 +1370,22 @@ module Version =
       string_attrib "method" (match m with `Get ->  "get" | `Post -> "post")
     let a_enctype = string_attrib "enctype"
 
+    let a_ismap `Ismap = string_attrib "ismap" "ismap"
+
     let a_checked `Checked = string_attrib "checked" "checked"
     let a_disabled `Disabled = string_attrib "disabled" "disabled"
     let a_readonly `Readonly = string_attrib "readonly" "readonly"
     let a_maxlength = int_attrib "maxlength"
     let a_name = string_attrib "name"
 
+   let a_span = int_attrib "span"
 
+    let a_value_type it =
+      string_attrib "valuetype"
+        (match it with
+        | `Data -> "data"
+        | `Ref -> "ref"
+        | `Object -> "object")
 
 (* XHTML 1.0 allows the name attribute for more elements:*)
     let a_name_01_00 = string_attrib "name"
@@ -1442,6 +1511,9 @@ module Version =
     type ('a, 'b) nullary = ?a:('a attrib list) -> unit -> 'b elt
     type ('a, 'b, 'c) unary = ?a:('a attrib list) -> 'b elt -> 'c elt
     type ('a, 'b, 'c, 'd) binary = ?a:('a attrib list) -> 'b elt -> 'c elt -> 'd elt
+(* CH *)
+    type ('a, 'b, 'c, 'd, 'e, 'f) quadry= ?a:('a attrib list) -> 'b elt -> 'c elt -> 'd elt -> 'e elt -> 'f elt
+(* CH *)
     type ('a, 'b, 'c) star = ?a:('a attrib list) -> 'b elt list -> 'c elt
     type ('a, 'b, 'c) plus = ?a:('a attrib list) -> 'b elt -> 'b elt list -> 'c elt
 
@@ -1452,6 +1524,11 @@ module Version =
     let star tag ?a elts = XML.node ?a tag elts
     let plus tag ?a elt elts = XML.node ?a tag (elt :: elts)
 
+(* CH *)
+    let quadry tag ?a elt1 elt2 elt3 elt4 = XML.node ?a tag [elt1; elt2; elt3; elt4]
+(* CH *)
+
+
     module STRUCTURE =
       struct
         type t = [ `Body | `Head | `Html | `Title ]
@@ -1580,12 +1657,17 @@ module Version =
     type i18nclass = [ `Bdo ]
     type shape = [ `Rect | `Circle | `Poly | `Default ]
 
-    let bdo = star "bdo"
 
     let a_datetime = string_attrib "datetime"
+
     let a_dir d =
       string_attrib "dir" (match d with `Rtl -> "rtl" | `Ltr -> "ltr")
 
+(* CH *)
+    let bdo ~dir ?(a = []) elts =
+      XML.node ~a:(a_dir dir :: a) "bdo" elts
+(* CH *)
+
     let a_shape d =
       string_attrib "shape"
         (match d with
@@ -1613,7 +1695,6 @@ module Version =
     let noscript = plus "noscript"
 (* VB *)
 
-
     module FORMS =
       struct
         type t = [ `Option ]
@@ -1708,11 +1789,8 @@ module Version =
     let tfoot = plus "tfoot"
 
     let object_ = star "object"
-
-    module IMAGE =
-      struct
-        type inline = [ `Img ]
-      end
+    let param ~name ?(a = []) () =
+      XML.leaf ~a:(a_name name :: a) "param"
 
     let img ~src ~alt ?(a = []) () =
       XML.leaf ~a:(a_src src :: a_alt alt :: a) "img"
@@ -1755,6 +1833,21 @@ module Version =
     let base ~href () =
       XML.leaf ~a:[a_href href] "base"
 
+    let ruby_simple1 = binary "ruby"
+    let ruby_simple2 = quadry "ruby"
+    let ruby_complex = binary "ruby"
+
+    let rbc = plus "rbc"
+    let rtc = plus "rtc"
+    let rtc_complex= plus "rtc"
+    let rb = star "rb"
+    let rt = star "rt"
+    let rt_complex = star "rt"
+    let rp = star "rp"
+
+    let a_rbspan = int_attrib "rbspan"
+
+
 (* VB *)
     type edit = [ `Ins | `Del ]
     type scripttag = [ `Script | `Noscript ]
@@ -1768,6 +1861,15 @@ module Version =
 
 (* VB *)
 
+(* CH *)
+    module RUBY = struct
+      type inline = [ `Ruby_simple1 | `Ruby_simple2 | `Ruby_complex ]
+      type flow =  inline 
+    end
+
+    type no_ruby_inline = [ TEXT.inline | PRESENTATION.inline | HYPERTEXT.inline | SPECIAL.inline | FORMS.inline | i18nclass ]
+    type no_ruby_content = [ `PCDATA | no_ruby_inline | misc ]
+(* CH *)
 
     type block =
         [ TEXT.block | PRESENTATION.block | FORMS.block | TABLES.block | SPECIAL.block | TEXT.heading | LIST.list | misc ]
@@ -1775,30 +1877,31 @@ module Version =
         [ TEXT.block | PRESENTATION.block | TABLES.block | TEXT.heading | LIST.list | misc ]
 
     type flow =
-        [ TEXT.flow | HYPERTEXT.flow | LIST.flow | FORMS.flow | TABLES.flow | PRESENTATION.flow | SPECIAL.flow | i18nclass | misc ]
+        [ TEXT.flow | HYPERTEXT.flow | LIST.flow | FORMS.flow | TABLES.flow | PRESENTATION.flow | SPECIAL.flow | i18nclass | misc | RUBY.flow ]
     type flow_sans_table =
-        [ TEXT.flow | HYPERTEXT.flow | LIST.flow | FORMS.flow | PRESENTATION.flow | SPECIAL.flow | i18nclass | misc ]
+        [ TEXT.flow | HYPERTEXT.flow | LIST.flow | FORMS.flow | PRESENTATION.flow | SPECIAL.flow | i18nclass | misc | RUBY.flow ]
 
     type inline =
         [ TEXT.inline | HYPERTEXT.inline | PRESENTATION.inline
-        | FORMS.inline | IMAGE.inline | SPECIAL.inline | i18nclass | misc ]
+        | FORMS.inline | SPECIAL.inline | i18nclass | misc | RUBY.inline ]
 
     type inline_sans_a_mix =
         [ TEXT.inline | PRESENTATION.inline
-        | FORMS.inline | IMAGE.inline | SPECIAL.inline | i18nclass | misc ]
+        | FORMS.inline | SPECIAL.inline | i18nclass | misc | RUBY.inline ]
 
     type buttoncontent = (* VB *)
         [ TEXT.inline | PRESENTATION.inline
-      | IMAGE.inline | SPECIAL.inline | i18nclass | block_sans_form ]
+        | SPECIAL.inline | i18nclass | block_sans_form ]
     type precontent = (* VB *)
         [ TEXT.inline | HYPERTEXT.inline | `Tt | `I | `B | `Script | `Map |
           i18nclass ]
     type inline_sans_label =
         [ TEXT.inline | HYPERTEXT.inline | PRESENTATION.inline
-        | FORMS.inline_sans_label | IMAGE.inline | SPECIAL.inline | i18nclass | misc ]
+        | FORMS.inline_sans_label | SPECIAL.inline | i18nclass | misc ]
 
     type heading = TEXT.heading
 
+
     (* I/O *)
 
     let compose_doctype dt args =
