@@ -26,7 +26,7 @@ val ocsigenadv : string
    if you want to print with a syntax closer to html (not xml).
  *)
 val xhtml_print :
-    ?version:[< `HTML_v03_02 | `HTML_v04_01 | `XHTML_01_00 | `XHTML_01_01 > `XHTML_01_01 ] ->
+    ?version:[< XHTML.M.doctypes > `XHTML_01_01 ] ->
       ?width:int -> ?encode:(string -> string) ->
         ?html_compat:bool ->
           [ `Html ] XHTML.M.elt -> string
@@ -35,26 +35,13 @@ val xhtml_print :
    [html_compat] is an option to set
    if you want to print with a syntax closer to html (not xml). *)
 val xhtml_list_print :
-    ?version:[< `HTML_v03_02 | `HTML_v04_01 | `XHTML_01_00 | `XHTML_01_01 > `XHTML_01_01 ] ->
+    ?version:[< XHTML.M.doctypes > `XHTML_01_01 ] ->
       ?width:int -> ?encode:(string -> string) ->
         ?html_compat:bool ->
           'a XHTML.M.elt list -> string
 
-(** Ocsigen's pretty printer for xhtml. [html_compat] is an option to set
-   if you want to print with a syntax closer to html (not xml).
- *)
-val xhtml_stream :
-    ?version:[< `HTML_v03_02 | `HTML_v04_01 | `XHTML_01_00 | `XHTML_01_01 > `XHTML_01_01 ] ->
-      ?width:int -> ?encode:(string -> string) ->
-        ?html_compat:bool ->
-          [ `Html ] XHTML.M.elt -> string Ocsigen_stream.t
-
-(** Ocsigen's pretty printer for xhtml portions.
-   [html_compat] is an option to set
-   if you want to print with a syntax closer to html (not xml). *)
-val xhtml_list_stream :
-    ?version:[< `HTML_v03_02 | `HTML_v04_01 | `XHTML_01_00 | `XHTML_01_01 > `XHTML_01_01 ] ->
-      ?width:int -> ?encode:(string -> string) ->
-        ?html_compat:bool ->
-          'a XHTML.M.elt list -> string Ocsigen_stream.t
-
+(**/**)
+val emptytags : string list
+val blocktags : string list
+val semiblocktags : string list
+val taille_tab : int
